@@ -29,60 +29,52 @@ import logging
 logger = logging.getLogger(__name__)
 
 import decimal
+import traceback
 
 from copy import copy
 from weakref import WeakKeyDictionary
-from collections import deque
+from collections import deque, OrderedDict
 from inspect import isclass
 from itertools import chain
 
-from spyne import BODY_STYLE_BARE, BODY_STYLE_WRAPPED, BODY_STYLE_EMPTY
 from spyne import const
-from spyne.const import xml_ns
+from spyne.const.xml import PREFMAP
 
-from spyne.model import Point
-from spyne.model import Unicode
-from spyne.model import PushBase
-from spyne.model import ModelBase
-from spyne.model import json, xml, msgpack, table
-from spyne.model._base import apply_pssm
+from spyne.model import Point, Unicode, PushBase, ModelBase
+from spyne.model._base import PSSM_VALUES, apply_pssm
 from spyne.model.primitive import NATIVE_MAP
 
-from spyne.util import six
-from spyne.util import memoize
-from spyne.util import memoize_id
-from spyne.util import sanitize_args
+from spyne.util import six, memoize, memoize_id, sanitize_args, \
+    memoize_ignore_none
+from spyne.util.color import YEL
 from spyne.util.meta import Prepareable
 from spyne.util.odict import odict
 from spyne.util.six import add_metaclass, with_metaclass, string_types
 
-
-PSSM_VALUES = {'json': json, 'xml': xml, 'msgpack': msgpack, 'table': table}
-
-
-def _is_under_pydev_debugger():
-    import inspect
-    for frame in inspect.stack():
-        if frame[1].endswith("pydevd.py"):
-            return True
-    return False
+# FIXME: for backwards compatibility, to be removed in Spyne 3
+# noinspection PyUnresolvedReferences
+from spyne.model import json, jsonb, xml, msgpack, table
 
 
 def _get_flat_type_info(cls, retval):
     assert isinstance(retval, TypeInfo)
     parent = getattr(cls, '__extends__', None)
-    if parent != None:
+    if not (parent is None):
         _get_flat_type_info(parent, retval)
     retval.update(cls._type_info)
-    retval.alt.update(cls._type_info_alt) # FIXME: move to cls._type_info.alt
+    retval.alt.update(cls._type_info_alt)  # FIXME: move to cls._type_info.alt
+    retval.attrs.update({k: v for (k, v) in cls._type_info.items()
+                                                if issubclass(v, XmlAttribute)})
     return retval
 
 
 class TypeInfo(odict):
     def __init__(self, *args, **kwargs):
         super(TypeInfo, self).__init__(*args, **kwargs)
+
         self.attributes = {}
-        self.alt = {}
+        self.alt = OrderedDict()
+        self.attrs = OrderedDict()
 
     def __setitem__(self, key, val):
         assert isinstance(key, string_types)
@@ -123,7 +115,7 @@ class XmlModifier(ModelBase):
         if cls.__namespace__ is None:
             cls.__namespace__ = cls.type.get_namespace()
 
-        if cls.__namespace__ in xml_ns.const_prefmap:
+        if cls.__namespace__ in PREFMAP:
             cls.__namespace__ = default_ns
 
     @classmethod
@@ -152,9 +144,9 @@ class XmlData(XmlModifier):
     def marshall(cls, prot, name, value, parent_elt):
         if value is not None:
             if len(parent_elt) == 0:
-                parent_elt.text = prot.to_string(cls.type, value)
+                parent_elt.text = prot.to_bytes(cls.type, value)
             else:
-                parent_elt[-1].tail = prot.to_string(cls.type, value)
+                parent_elt[-1].tail = prot.to_bytes(cls.type, value)
 
     @classmethod
     def get_type_name(cls):
@@ -178,17 +170,13 @@ class XmlData(XmlModifier):
 
 
 class XmlAttribute(XmlModifier):
-    """Items which are marshalled as attributes of the parent element. If
-    ``attribute_of`` is passed, it's marshalled as the attribute of the element
-    with given name.
-    """
+    """Items which are marshalled as attributes of the parent element."""
 
-    def __new__(cls, type_, use=None, ns=None, attribute_of=None):
+    def __new__(cls, type_, use=None, ns=None):
         retval = super(XmlAttribute, cls).__new__(cls, type_, ns)
         retval._use = use
         if retval.type.Attributes.min_occurs > 0 and retval._use is None:
             retval._use = 'required'
-        retval.attribute_of = attribute_of
         return retval
 
 
@@ -218,7 +206,7 @@ class SelfReference(object):
 
     @classmethod
     def customize(cls, *args, **kwargs):
-        args = chain(args, cls.customize_args)
+        args = list(chain(args, cls.customize_args))
         kwargs = dict(chain(kwargs.items(), cls.customize_kwargs.items()))
         if cls.__orig__ is None:
             cls.__orig__ = cls
@@ -335,10 +323,20 @@ def _get_type_info(cls, cls_name, cls_bases, cls_dict, attrs):
 
         class_fields = []
         for k, v in cls_dict.items():
-            if not k.startswith('_'):
-                v = _get_spyne_type(cls_name, k, v)
-                if v is not None:
-                    class_fields.append((k, v))
+            if k.startswith('_'):
+                continue
+
+            if isinstance(v, tuple) and len(v) == 1 and \
+                                 _get_spyne_type(cls_name, k, v[0]) is not None:
+                logger.warning(YEL("There seems to be a stray comma in the"
+                                   "definition of '%s.%s'.", cls_name, k))
+
+            v = _get_spyne_type(cls_name, k, v)
+
+            if v is None:
+                continue
+
+            class_fields.append((k, v))
 
         _type_info.update(class_fields)
 
@@ -348,7 +346,8 @@ def _get_type_info(cls, cls_name, cls_bases, cls_dict, attrs):
         if not isinstance(_type_info, TypeInfo):
             _type_info = cls_dict['_type_info'] = TypeInfo(_type_info)
 
-    _type_info.update(mixin)
+    for k, v in reversed(mixin.items()):
+        _type_info.insert(0, (k, v))
 
     return _type_info
 
@@ -359,63 +358,24 @@ class _MethodsDict(dict):
 
         self._processed = False
 
-    def sanitize(self, cls):
-        # sanitize is called on every customization, so we make sure it's run
-        # only once in this class' lifetime.
-        if self._processed:
-            return
-
-        self._processed = True
-
-        for d in self.values():
-            d.parent_class = cls
-
-            if d.in_message_name_override:
-                d.in_message.__type_name__ = '%s.%s' % \
-                          (cls.get_type_name(), d.in_message.get_type_name())
-
-            if d.body_style is BODY_STYLE_WRAPPED or d.out_message_name_override:
-                d.out_message.__type_name__ = '%s.%s' % \
-                          (cls.get_type_name(), d.out_message.get_type_name())
-
-            if d.body_style in (BODY_STYLE_BARE, BODY_STYLE_EMPTY):
-                # The method only needs the primary key(s) and shouldn't
-                # complain when other mandatory fields are missing.
-                d.in_message = cls.novalidate_freq()
-                d.body_style = BODY_STYLE_BARE
-
-            else:
-                d.in_message.insert_field(0, 'self', cls.novalidate_freq())
-                d.body_style = BODY_STYLE_WRAPPED
-
-                if issubclass(d.in_message, ComplexModelBase):
-                    for k, v in d.in_message._type_info.items():
-                        # SelfReference is replaced by descriptor.in_message
-                        # itself. However, in the context of mrpc, SelfReference
-                        # means parent class. here, we do that substitution.
-                        # It's a safe hack but a hack nevertheless.
-                        if v is d.in_message:
-                            d.in_message._type_info[k] = cls
-
-            # Same as above, for the output type.
-            if issubclass(d.out_message, ComplexModelBase):
-                for k, v in d.out_message._type_info.items():
-                    if v is d.out_message:
-                        d.out_message._type_info[k] = cls
-
-            if d.patterns is not None and not d.no_self:
-                d.name = '.'.join((cls.get_type_name(), d.name))
-                for p in d.patterns:
-                    if p.address is None:
-                        p.address = d.name
-
 
 def _gen_methods(cls, cls_dict):
     methods = _MethodsDict()
     for k, v in cls_dict.items():
         if not k.startswith('_') and hasattr(v, '_is_rpc'):
+            logger.debug("Registering %s as member method for %r", k, cls)
+            assert cls is not None
+
+            # generate method descriptor from information in the decorator
             descriptor = v(_default_function_name=k, _self_ref_replacement=cls)
+
+            # strip the decorator and put the original function in the class
             setattr(cls, k, descriptor.function)
+
+            # modify the descriptor with user-supplied class
+            if cls.Attributes.method_config_do is not None:
+                descriptor = cls.Attributes.method_config_do(descriptor)
+
             methods[k] = descriptor
 
     return methods
@@ -462,10 +422,12 @@ def _sanitize_sqlalchemy_parameters(cls_dict, attrs):
 
 
 def _sanitize_type_info(cls_name, _type_info, _type_info_alt):
-    # make sure _type_info contents are sane
+    """Make sure _type_info contents are sane"""
+
     for k, v in _type_info.items():
         if not isinstance(k, six.string_types):
             raise ValueError("Invalid class key", k)
+
         if not isclass(v):
             raise ValueError(v)
 
@@ -498,19 +460,63 @@ def _sanitize_type_info(cls_name, _type_info, _type_info_alt):
             key = sub_name
             if sub_ns in _type_info:
                 raise Exception("%r is already defined: %r" %
-                                                        (key, _type_info[key]))
+                                                         (key, _type_info[key]))
             _type_info_alt[key] = v, k
 
         elif sub_name is None:
             key = "{%s}%s" % (sub_ns, k)
             if key in _type_info:
                 raise Exception("%r is already defined: %r" %
-                                                        (key, _type_info[key]))
+                                                         (key, _type_info[key]))
             _type_info_alt[key] = v, k
 
 
+D_EXC = dict(exc=True)
+
+
 def _process_child_attrs(cls, retval, kwargs):
+    child_attrs = copy(kwargs.get('child_attrs', None))
     child_attrs_all = kwargs.get('child_attrs_all', None)
+    child_attrs_noexc = copy(kwargs.get('child_attrs_noexc', None))
+
+    # add exc=False to child_attrs_noexc
+    if child_attrs_noexc is not None:
+        # if there is _noexc, make sure that child_attrs_all is also used to
+        # exclude exclude everything else first
+        if child_attrs_all is None:
+            child_attrs_all = D_EXC
+
+        else:
+            if 'exc' in child_attrs_all and child_attrs_all['exc'] != D_EXC:
+                logger.warning("Overriding child_attrs_all['exc'] to True "
+                                                                  "for %r", cls)
+
+            child_attrs_all.update(D_EXC)
+
+        # update child_attrs_noexc with exc=False
+        for k, v in child_attrs_noexc.items():
+            if 'exc' in v:
+                logger.warning("Overriding 'exc' for %s.%s from "
+                         "child_attrs_noexc with False", cls.get_type_name(), k)
+
+            v['exc'] = False
+
+        # update child_attrs with data from child_attrs_noexc
+        if child_attrs is None:
+            child_attrs = child_attrs_noexc
+
+        else:
+            # update with child_attrs_noexc with exc=False
+            if child_attrs is None:
+                child_attrs = dict()
+
+            for k, v in child_attrs_noexc.items():
+                if k in child_attrs:
+                    logger.warning("Overriding child_attrs for %s.%s from "
+                                    "child_attrs_noexc", cls.get_type_name(), k)
+
+                child_attrs[k] = v
+
     if child_attrs_all is not None:
         ti = retval._type_info
         logger.debug("processing child_attrs_all for %r", cls)
@@ -520,11 +526,10 @@ def _process_child_attrs(cls, retval, kwargs):
 
         if retval.__extends__ is not None:
             retval.__extends__ = retval.__extends__.customize(
-                                            child_attrs_all=child_attrs_all)
+                                                child_attrs_all=child_attrs_all)
 
         retval.Attributes._delayed_child_attrs_all = child_attrs_all
 
-    child_attrs = copy(kwargs.get('child_attrs', None))
     if child_attrs is not None:
         ti = retval._type_info
         logger.debug("processing child_attrs for %r", cls)
@@ -537,15 +542,101 @@ def _process_child_attrs(cls, retval, kwargs):
         base_fti = {}
         if retval.__extends__ is not None:
             retval.__extends__ = retval.__extends__.customize(
-                                                    child_attrs=child_attrs)
-            base_fti = retval.__extends__.get_flat_type_info(
-                                                         retval.__extends__)
+                                                        child_attrs=child_attrs)
+            base_fti = retval.__extends__.get_flat_type_info(retval.__extends__)
+
         for k, v in child_attrs.items():
             if k not in base_fti:
                 logger.debug("  child_attr delayed %r=%r", k, v)
                 retval.Attributes._delayed_child_attrs[k] = v
 
 
+def recust_selfref(selfref, cls):
+    if len(selfref.customize_args) > 0 or len(selfref.customize_kwargs) > 0:
+        logger.debug("Replace self reference with %r with *%r and **%r",
+                          cls, selfref.customize_args, selfref.customize_kwargs)
+        return cls.customize(*selfref.customize_args,
+                                                     **selfref.customize_kwargs)
+    logger.debug("Replace self reference with %r", cls)
+    return cls
+
+
+def _set_member_default(inst, key, cls, attr):
+    def_val = attr.default
+    def_fac = attr.default_factory
+
+    if def_fac is None and def_val is None:
+        return False
+
+    if def_fac is not None:
+        if six.PY2 and hasattr(def_fac, 'im_func'):
+            # unbound-method error workaround. huh.
+            def_fac = def_fac.im_func
+
+        dval = def_fac()
+
+        # should not check for read-only for default values
+        setattr(inst, key, dval)
+
+        return True
+
+    if def_val is not None:
+        # should not check for read-only for default values
+        setattr(inst, key, def_val)
+
+        return True
+
+    assert False, "Invalid application state"
+
+
+def _is_sqla_array(cls, attr):
+    # inner object is complex
+    ret1 = issubclass(cls, Array) and \
+                              hasattr(cls.get_inner_type(), '_sa_class_manager')
+
+    # inner object is primitive
+    ret2 = issubclass(cls, Array) and attr.store_as is not None
+
+    # object is a bare array
+    ret3 = attr.max_occurs > 1 and hasattr(cls, '_sa_class_manager')
+
+    return ret1 or ret2 or ret3
+
+
+def _init_member(inst, key, cls, attr):
+    cls_getattr_ret = getattr(inst.__class__, key, None)
+
+    if isinstance(cls_getattr_ret, property) and cls_getattr_ret.fset is None:
+        return  # we skip read-only properties
+
+    if _set_member_default(inst, key, cls, attr):
+       return
+
+    # sqlalchemy objects do their own init.
+    if _is_sqla_array(cls, attr):
+        # except the attributes that sqlalchemy doesn't know about
+        if attr.exc_db:
+            setattr(inst, key, None)
+
+        elif attr.store_as is None:
+            setattr(inst, key, None)
+
+        return
+
+    # sqlalchemy objects do their own init.
+    if hasattr(inst.__class__, '_sa_class_manager'):
+        # except the attributes that sqlalchemy doesn't know about
+        if attr.exc_db:
+            setattr(inst, key, None)
+
+        elif issubclass(cls, ComplexModelBase) and attr.store_as is None:
+            setattr(inst, key, None)
+
+        return
+
+    setattr(inst, key, None)
+
+
 class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
     """This metaclass sets ``_type_info``, ``__type_name__`` and ``__extends__``
     which are going to be used for (de)serialization and schema generation.
@@ -575,37 +666,35 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
         _sanitize_type_info(cls_name, _type_info, _type_info_alt)
         _sanitize_sqlalchemy_parameters(cls_dict, attrs)
 
-        return super(ComplexModelMeta, cls).__new__(cls, cls_name, cls_bases,
-                                                    cls_dict)
+        return super(ComplexModelMeta, cls).__new__(cls,
+                                                  cls_name, cls_bases, cls_dict)
 
     def __init__(self, cls_name, cls_bases, cls_dict):
         type_info = self._type_info
 
         extends = self.__extends__
         if extends is not None and self.__orig__ is None:
-            eattr = extends.Attributes;
+            eattr = extends.Attributes
             if eattr._subclasses is None:
                 eattr._subclasses = []
             eattr._subclasses.append(self)
             if self.Attributes._subclasses is eattr._subclasses:
                 self.Attributes._subclasses = None
 
+        # sanitize fields
         for k, v in type_info.items():
+            # replace bare SelfRerefence
             if issubclass(v, SelfReference):
-                self._replace_field(k, self.customize(*v.customize_args,
-                                                          **v.customize_kwargs))
+                self._replace_field(k, recust_selfref(v, self))
 
+            # cache XmlData for easier access
             elif issubclass(v, XmlData):
                 if self.Attributes._xml_tag_body_as is None:
                     self.Attributes._xml_tag_body_as = [(k, v)]
                 else:
                     self.Attributes._xml_tag_body_as.append((k, v))
 
-            elif issubclass(v, XmlAttribute):
-                a_of = v.attribute_of
-                if a_of is not None:
-                    type_info.attributes[k] = type_info[a_of]
-
+            # replace SelfRerefence in arrays
             elif issubclass(v, Array):
                 v2, = v._type_info.values()
                 while issubclass(v2, Array):
@@ -613,8 +702,9 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
                     v2, = v2._type_info.values()
 
                 if issubclass(v2, SelfReference):
-                    v._set_serializer(self)
+                    v._set_serializer(recust_selfref(v2, self))
 
+        # apply field order
         # FIXME: Implement this better
         new_type_info = []
         for k, v in self._type_info.items():
@@ -628,18 +718,32 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
         assert len(self._type_info) == len(new_type_info)
         self._type_info.keys()[:] = new_type_info
 
-        tn = self.Attributes.table_name
-        meta = self.Attributes.sqla_metadata
-        t = self.Attributes.sqla_table
+        # install checkers for validation on assignment
+        for k, v in self._type_info.items():
+            if not v.Attributes.validate_on_assignment:
+                continue
 
+            def _get_prop(self):
+                return self.__dict__[k]
+
+            def _set_prop(self, val):
+                if not (val is None or isinstance(val, v.Value)):
+                    raise ValueError("Invalid value %r, "
+                                 "should be an instance of %r" % (val, v.Value))
+
+                self.__dict__[k] = val
+
+            setattr(self, k, property(_get_prop, _set_prop))
+
+        # process member rpc methods
         methods = _gen_methods(self, cls_dict)
         if len(methods) > 0:
             self.Attributes.methods = methods
 
-        methods = self.Attributes.methods
-        if methods is not None:
-            assert isinstance(methods, _MethodsDict)
-            methods.sanitize(self)
+        # finalize sql table mapping
+        tn = self.Attributes.table_name
+        meta = self.Attributes.sqla_metadata
+        t = self.Attributes.sqla_table
 
         # For spyne objects reflecting an existing db table
         if tn is None:
@@ -670,24 +774,8 @@ class ComplexModelMeta(with_metaclass(Prepareable, type(ModelBase))):
     def __prepare__(mcs, name, bases, **kwds):
         return odict()
 
-    #
-    #  pydev debugger seems to secretly manipulate class dictionaries without
-    # doing proper checks -- not always do a class with a metaclass with a
-    # __prepare__ has to have it. this code has a very specific workaround to
-    # make debugging under pydev for python 3.x work.
-    #
-    # see https://github.com/arskom/spyne/issues/432 for details
-    #
-    # this can be removed once pydev fixes secret calls to class dict's
-    # __delitem__
-    if _is_under_pydev_debugger():
-        @classmethod
-        def __prepare__(mcs, name, bases, **kwds):
-            return odict((('__class__', mcs),))
-        print("ComplexModelMeta.__prepare__ substitution for PyDev successful")
 
-
-_is_array = lambda v: issubclass(v, Array) or (v.Attributes.min_occurs > 1)
+_is_array = lambda v: issubclass(v, Array) or (v.Attributes.max_occurs > 1)
 
 
 class ComplexModelBase(ModelBase):
@@ -703,7 +791,7 @@ class ComplexModelBase(ModelBase):
         store_as = None
         """Method for serializing to persistent storage. One of %r. It makes
         sense to specify this only when this object is a child of another
-        ComplexModel sublass.""" % (PSSM_VALUES,)
+        ComplexModel subclass.""" % (PSSM_VALUES,)
 
         sqla_metadata = None
         """None or :class:`sqlalchemy.MetaData` instance."""
@@ -768,7 +856,20 @@ class ComplexModelBase(ModelBase):
         """FIXME: document me yo."""
 
         methods = None
-        """FIXME: document me yo."""
+        """A dict of member RPC methods (typically marked with @mrpc)."""
+
+        method_config_do = None
+        """When not None, it's a callable that accepts a ``@mrpc`` method
+        descriptor and returns a modified version."""
+
+        not_wrapped = None
+        """When True, serializes to non-wrapped object, overriding the protocol
+        flag."""
+
+        wrapped = None
+        """When True, serializes to a wrapped object, overriding the protocol
+        flag. When a str/bytes/unicode value, uses that value as key wrapper
+        object name."""
 
         _variants = None
         _xml_tag_body_as = None
@@ -778,17 +879,24 @@ class ComplexModelBase(ModelBase):
 
     def __init__(self, *args, **kwargs):
         cls = self.__class__
+        cls_attr = cls.Attributes
         fti = cls.get_flat_type_info(cls)
 
         if cls.__orig__ is not None:
-            logger.warning("%r seems to be a customized class. It is not "
-                      "supposed to be instantiated. You have been warned.", cls)
+            logger.warning("%r(0x%X) seems to be a customized class. It is not "
+                    "supposed to be instantiated. You have been warned.",
+                                                                   cls, id(cls))
+            logger.debug(traceback.format_stack())
 
-        if cls.Attributes._xml_tag_body_as is not None:
+        if cls_attr._xml_tag_body_as is not None:
             for arg, (xtba_key, xtba_type) in \
-                                     zip(args, cls.Attributes._xml_tag_body_as):
+                                           zip(args, cls_attr._xml_tag_body_as):
+
                 if xtba_key is not None and len(args) == 1:
-                    self._safe_set(xtba_key, arg, xtba_type)
+                    attr = xtba_type.Attributes
+                    _init_member(self, xtba_key, xtba_type, attr)
+                    self._safe_set(xtba_key, arg, xtba_type,
+                                                           xtba_type.Attributes)
                 elif len(args) > 0:
                     raise TypeError(
                                 "Positional argument is only for ComplexModels "
@@ -796,41 +904,12 @@ class ComplexModelBase(ModelBase):
                                 "arguments in any other case.")
 
         for k, v in fti.items():
+            attr = v.Attributes
+            if not k in self.__dict__:
+                _init_member(self, k, v, attr)
+
             if k in kwargs:
-                self._safe_set(k, kwargs[k], v)
-
-            elif not k in self.__dict__:
-                attr = v.Attributes
-                def_val = attr.default
-                def_fac = attr.default_factory
-
-                if def_fac is not None:
-                    if six.PY2 and hasattr(def_fac, 'im_func'):
-                        # unbound-method error workaround. huh.
-                        def_fac = def_fac.im_func
-                    dval = def_fac()
-
-                    # should not check for read-only for default values
-                    setattr(self, k, dval)
-
-                elif def_val is not None:
-                    # should not check for read-only for default values
-                    setattr(self, k, def_val)
-
-                # sqlalchemy objects do their own init.
-                elif hasattr(cls, '_sa_class_manager'):
-                    # except the attributes that sqlalchemy doesn't know about
-                    if v.Attributes.exc_table:
-                        try:
-                            setattr(self, k, None)
-                        except AttributeError: # it could be a read-only property
-                            pass
-
-                    elif issubclass(v, ComplexModelBase) and \
-                                                  v.Attributes.store_as is None:
-                        setattr(self, k, None)
-                else:
-                    setattr(self, k, None)
+                self._safe_set(k, kwargs[k], v, attr)
 
     def __len__(self):
         return len(self._type_info)
@@ -852,8 +931,8 @@ class ComplexModelBase(ModelBase):
                     for k in self.__class__.get_flat_type_info(self.__class__)
                     if self.__dict__.get(k, None) is not None]))
 
-    def _safe_set(self, key, value, t):
-        if t.Attributes.read_only:
+    def _safe_set(self, key, value, t, attrs):
+        if attrs.read_only:
             return False
 
         try:
@@ -865,15 +944,25 @@ class ComplexModelBase(ModelBase):
 
         return True
 
+    @classmethod
+    def get_identifiers(cls):
+        for k, v in cls.get_flat_type_info(cls).items():
+            if getattr(v.Attributes, 'primary_key', None):
+                yield k, v
+
+    @classmethod
+    def get_primary_keys(cls):
+        return cls.get_identifiers()
+
     def as_dict(self):
         """Represent object as dict.
 
         Null values are omitted from dict representation to support optional
         not nullable attributes.
-
         """
+
         return dict((
-            (k, getattr(self, k)) for k in self.get_flat_type_info(self)
+            (k, getattr(self, k)) for k in self.get_flat_type_info(self.__class__)
             if getattr(self, k) is not None
         ))
 
@@ -897,14 +986,24 @@ class ComplexModelBase(ModelBase):
         # (as the members are declared and passed around as sequences of
         # arguments, unlike dictionaries in a regular class definition).
         if isinstance(value, list) or isinstance(value, tuple):
-            assert len(value) <= len(cls._type_info)
+            keys = cls.get_flat_type_info(cls).keys()
+
+            if not len(value) <= len(keys):
+                logger.error("\n\tcls: %r" "\n\tvalue: %r" "\n\tkeys: %r",
+                                                               cls, value, keys)
+                raise ValueError("Impossible sequence to instance conversion")
 
             cls_orig = cls
             if cls.__orig__ is not None:
                 cls_orig = cls.__orig__
-            inst = cls_orig()
 
-            keys = cls._type_info.keys()
+            try:
+                inst = cls_orig()
+
+            except Exception as e:
+                logger.error("Error instantiating %r: %r", cls_orig, e)
+                raise
+
             for i in range(len(value)):
                 setattr(inst, keys[i], value[i])
 
@@ -914,7 +1013,7 @@ class ComplexModelBase(ModelBase):
                 cls_orig = cls.__orig__
             inst = cls_orig()
 
-            for k in cls._type_info:
+            for k in cls.get_flat_type_info(cls):
                 setattr(inst, k, value.get(k, None))
 
         else:
@@ -923,7 +1022,7 @@ class ComplexModelBase(ModelBase):
         return inst
 
     @classmethod
-    def get_deserialization_instance(cls):
+    def get_deserialization_instance(cls, ctx):
         """Get an empty native type so that the deserialization logic can set
         its attributes.
         """
@@ -943,7 +1042,7 @@ class ComplexModelBase(ModelBase):
         return retval
 
     @staticmethod
-    @memoize
+    @memoize_ignore_none
     def get_flat_type_info(cls):
         """Returns a _type_info dict that includes members from all base
         classes.
@@ -1064,11 +1163,11 @@ class ComplexModelBase(ModelBase):
         could cause memory leaks. You have been warned.
         """
 
-        store_as = apply_pssm(kwargs.get('store_as', None), PSSM_VALUES)
+        store_as = apply_pssm(kwargs.get('store_as', None))
         if store_as is not None:
             kwargs['store_as'] = store_as
 
-        cls_name, cls_bases, cls_dict = cls._s_customize(cls, **kwargs)
+        cls_name, cls_bases, cls_dict = cls._s_customize(**kwargs)
         cls_dict['__module__'] = cls.__module__
         if '__extends__' not in cls_dict:
             cls_dict['__extends__'] = cls.__extends__
@@ -1209,16 +1308,13 @@ class ComplexModelBase(ModelBase):
     def init_from(cls, other, **kwargs):
         retval = (cls if cls.__orig__ is None else cls.__orig__)()
 
-        for k, v in cls._type_info.items():
-            if v.Attributes.read_only:
-                continue
-
+        for k, v in cls.get_flat_type_info(cls).items():
             try:
                 if k in kwargs:
-                    setattr(retval, k, kwargs[k])
+                    retval._safe_set(k, kwargs[k], v, v.Attributes)
 
                 elif hasattr(other, k):
-                    setattr(retval, k, getattr(other, k))
+                    retval._safe_set(k, getattr(other, k), v, v.Attributes)
 
             except AttributeError as e:
                 logger.warning("Error setting %s: %r", k, e)
@@ -1226,10 +1322,15 @@ class ComplexModelBase(ModelBase):
         return retval
 
     @classmethod
-    def __respawn__(cls, ctx=None):
+    def __respawn__(cls, ctx=None, filters=None):
         if ctx is not None and ctx.in_object is not None and \
                                                          len(ctx.in_object) > 0:
-            return ctx.in_object[0]
+            retval = next(iter(ctx.in_object))
+            if retval is not None:
+                return retval
+
+        if ctx.descriptor.default_on_null:
+            return cls.get_deserialization_instance(ctx)
 
 
 @add_metaclass(ComplexModelMeta)
@@ -1303,6 +1404,8 @@ class Array(ComplexModelBase):
 
         del kwargs['serializer_attrs']
 
+        logger.debug('Pass serializer attrs %r', serializer_attrs)
+
         serializer, = cls._type_info.values()
         return cls(serializer.customize(**serializer_attrs)).customize(**kwargs)
 
@@ -1340,7 +1443,7 @@ class Array(ComplexModelBase):
         if cls.__namespace__ is None:
             cls.__namespace__ = serializer.get_namespace()
 
-        if cls.__namespace__ in xml_ns.const_prefmap:
+        if cls.__namespace__ in PREFMAP:
             cls.__namespace__ = default_ns
 
         return ComplexModel.resolve_namespace(cls, default_ns, tags)
@@ -1355,9 +1458,13 @@ class Array(ComplexModelBase):
         return inst
 
     @classmethod
-    def get_deserialization_instance(cls):
+    def get_deserialization_instance(cls, ctx):
         return []
 
+    @classmethod
+    def get_inner_type(cls):
+        return next(iter(cls._type_info.values()))
+
 
 class Iterable(Array):
     """This class generates a ``ComplexModel`` child that has one attribute that
@@ -1375,10 +1482,20 @@ class Iterable(Array):
         logged = False
 
     class Push(PushBase):
+        """The push interface to the `Iterable`.
+
+        Anything append()'ed to a `Push` instance is serialized and written to
+        outgoing stream immediately.
+
+        When using Twisted, Push callbacks are called from the reactor thread if
+        the instantiation is done in a reactor thread. Otherwise, callbacks are
+        called by `deferToThread`. Make sure to avoid relying on thread-local
+        stuff as `deferToThread` is not guaranteed to restore original thread
+        context.
+        """
         pass
 
 
-@memoize
 def TTableModelBase():
     from spyne.store.relational import add_column
 
@@ -1409,18 +1526,22 @@ def TTableModelBase():
 
 
 # this has docstring repeated in the documentation at reference/model/complex.rst
-@memoize_id
-def TTableModel(metadata=None):
+def TTableModel(metadata=None, base=None, metaclass=None):
     """A TableModel template that generates a new TableModel class for each
     call. If metadata is not supplied, a new one is instantiated.
     """
 
     from sqlalchemy import MetaData
 
-    @add_metaclass(ComplexModelMeta)
-    class TableModel(TTableModelBase()):
+    if base is None:
+        base = TTableModelBase()
+    if metaclass is None:
+        metaclass = ComplexModelMeta
+
+    @add_metaclass(metaclass)
+    class TableModel(base):
         class Attributes(ComplexModelBase.Attributes):
-            sqla_metadata = metadata or MetaData()
+            sqla_metadata = metadata if metadata is not None else MetaData()
 
     return TableModel
 
@@ -1428,7 +1549,7 @@ def TTableModel(metadata=None):
 def Mandatory(cls, **_kwargs):
     """Customizes the given type to be a mandatory one. Has special cases for
     :class:`spyne.model.primitive.Unicode` and
-    :class:`spyne.model.complex.Array`\.
+    :class:`spyne.model.complex.Array`\\.
     """
 
     kwargs = dict(min_occurs=1, nillable=False)
