@@ -38,6 +38,7 @@ from spyne.model.primitive.xml import Token
 from spyne.model.primitive.xml import NMToken
 from spyne.model.primitive.xml import Name
 from spyne.model.primitive.xml import NCName
+from spyne.model.primitive.xml import QName
 from spyne.model.primitive.xml import Language
 from spyne.model.primitive.xml import NormalizedString
 
@@ -70,6 +71,7 @@ from spyne.model.primitive.number import Int
 from spyne.model.primitive.number import Integer64
 from spyne.model.primitive.number import Long
 from spyne.model.primitive.number import Integer
+from spyne.model.primitive.number import NumberLimitsWarning
 
 from spyne.model.primitive.number import UnsignedInteger8
 from spyne.model.primitive.number import UnsignedByte
@@ -80,7 +82,6 @@ from spyne.model.primitive.number import UnsignedInt
 from spyne.model.primitive.number import UnsignedInteger64
 from spyne.model.primitive.number import UnsignedLong
 from spyne.model.primitive.number import NonNegativeInteger # Xml Schema calls it so
-from spyne.model.primitive.number import PositiveInteger
 from spyne.model.primitive.number import UnsignedInteger
 
 from spyne.model.primitive.network import MacAddress
