@@ -17,20 +17,21 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301
 #
 
-from __future__ import print_function
+from __future__ import print_function, unicode_literals
 
 import logging
 logger = logging.getLogger(__name__)
 
+import re
 import uuid
 import errno
 
-from os.path import isabs, join
+from os.path import isabs, join, abspath
 from collections import deque
-from datetime import timedelta, datetime
+from datetime import datetime
 from decimal import Decimal as D
 from mmap import mmap, ACCESS_READ
-from time import mktime
+from time import mktime, strftime
 
 try:
     from lxml import etree
@@ -40,23 +41,18 @@ except ImportError:
     html = None
 
 from spyne.protocol._base import ProtocolMixin
-
 from spyne.model import ModelBase, XmlAttribute, SimpleModel, Null, \
     ByteArray, File, ComplexModelBase, AnyXml, AnyHtml, Unicode, Decimal, \
-    Double, Integer, Time, DateTime, Uuid, Duration, Boolean
-
+    Double, Integer, Time, DateTime, Uuid, Duration, Boolean, AnyDict, \
+    AnyUri, PushBase, Date
 from spyne.const.http import HTTP_400, HTTP_401, HTTP_404, HTTP_405, HTTP_413, \
     HTTP_500
-
 from spyne.error import Fault, InternalError, ResourceNotFoundError, \
     RequestTooLongError, RequestNotAllowed, InvalidCredentialsError
-
 from spyne.model.binary import binary_encoding_handlers, \
     BINARY_ENCODING_USE_DEFAULT
 
 from spyne.util import six
-from spyne.model.binary import Attachment  # DEPRECATED
-
 from spyne.util.cdict import cdict
 
 
@@ -86,69 +82,69 @@ class OutProtocolBase(ProtocolMixin):
     """
 
     def __init__(self, app=None, mime_type=None, ignore_uncap=False,
-                                   ignore_wrappers=False, binary_encoding=None):
+             ignore_wrappers=False, binary_encoding=None, string_encoding=None):
 
         super(OutProtocolBase, self).__init__(app=app, mime_type=mime_type,
-                                                ignore_wrappers=ignore_wrappers)
+            ignore_wrappers=ignore_wrappers,
+               binary_encoding=binary_encoding, string_encoding=string_encoding)
 
         self.ignore_uncap = ignore_uncap
         self.message = None
-        self.binary_encoding = binary_encoding
-
-        if self.binary_encoding is None:
-            self.binary_encoding = self.default_binary_encoding
 
         if mime_type is not None:
             self.mime_type = mime_type
 
-        self._to_string_handlers = cdict({
-            ModelBase: self.model_base_to_string,
-            File: self.file_to_string,
-            Time: self.time_to_string,
-            Uuid: self.uuid_to_string,
-            Null: self.null_to_string,
-            Double: self.double_to_string,
-            AnyXml: self.any_xml_to_string,
-            Unicode: self.unicode_to_string,
-            Boolean: self.boolean_to_string,
-            Decimal: self.decimal_to_string,
-            Integer: self.integer_to_string,
-            AnyHtml: self.any_html_to_string,
-            DateTime: self.datetime_to_string,
-            Duration: self.duration_to_string,
-            ByteArray: self.byte_array_to_string,
-            Attachment: self.attachment_to_string,
-            XmlAttribute: self.xmlattribute_to_string,
-            ComplexModelBase: self.complex_model_base_to_string,
+        self._to_bytes_handlers = cdict({
+            ModelBase: self.model_base_to_bytes,
+            File: self.file_to_bytes,
+            Time: self.time_to_bytes,
+            Uuid: self.uuid_to_bytes,
+            Null: self.null_to_bytes,
+            Date: self.date_to_bytes,
+            Double: self.double_to_bytes,
+            AnyXml: self.any_xml_to_bytes,
+            Unicode: self.unicode_to_bytes,
+            Boolean: self.boolean_to_bytes,
+            Decimal: self.decimal_to_bytes,
+            Integer: self.integer_to_bytes,
+            AnyHtml: self.any_html_to_bytes,
+            DateTime: self.datetime_to_bytes,
+            Duration: self.duration_to_bytes,
+            ByteArray: self.byte_array_to_bytes,
+            XmlAttribute: self.xmlattribute_to_bytes,
+            ComplexModelBase: self.complex_model_base_to_bytes,
         })
 
         self._to_unicode_handlers = cdict({
             ModelBase: self.model_base_to_unicode,
             File: self.file_to_unicode,
-            Time: self.time_to_string,
-            Uuid: self.uuid_to_string,
-            Null: self.null_to_string,
-            Double: self.double_to_string,
+            Time: self.time_to_unicode,
+            Date: self.date_to_unicode,
+            Uuid: self.uuid_to_unicode,
+            Null: self.null_to_unicode,
+            Double: self.double_to_unicode,
             AnyXml: self.any_xml_to_unicode,
-            Unicode: self.unicode_to_unicode,
-            Boolean: self.boolean_to_string,
-            Decimal: self.decimal_to_string,
-            Integer: self.integer_to_string,
+            AnyUri: self.any_uri_to_unicode,
+            AnyDict: self.any_dict_to_unicode,
             AnyHtml: self.any_html_to_unicode,
+            Unicode: self.unicode_to_unicode,
+            Boolean: self.boolean_to_unicode,
+            Decimal: self.decimal_to_unicode,
+            Integer: self.integer_to_unicode,
             # FIXME: Would we need a to_unicode for localized dates?
-            DateTime: self.datetime_to_string,
-            Duration: self.duration_to_string,
+            DateTime: self.datetime_to_unicode,
+            Duration: self.duration_to_unicode,
             ByteArray: self.byte_array_to_unicode,
             XmlAttribute: self.xmlattribute_to_unicode,
-            ComplexModelBase: self.complex_model_base_to_string,
+            ComplexModelBase: self.complex_model_base_to_unicode,
         })
 
-        self._to_string_iterable_handlers = cdict({
-            File: self.file_to_string_iterable,
-            ByteArray: self.byte_array_to_string_iterable,
-            ModelBase: self.model_base_to_string_iterable,
-            SimpleModel: self.simple_model_to_string_iterable,
-            ComplexModelBase: self.complex_model_to_string_iterable,
+        self._to_bytes_iterable_handlers = cdict({
+            File: self.file_to_bytes_iterable,
+            ByteArray: self.byte_array_to_bytes_iterable,
+            ModelBase: self.model_base_to_bytes_iterable,
+            SimpleModel: self.simple_model_to_bytes_iterable,
+            ComplexModelBase: self.complex_model_to_bytes_iterable,
         })
 
 
@@ -173,12 +169,16 @@ class OutProtocolBase(ProtocolMixin):
 
         if isinstance(fault, RequestTooLongError):
             return HTTP_413
+
         if isinstance(fault, ResourceNotFoundError):
             return HTTP_404
+
         if isinstance(fault, RequestNotAllowed):
             return HTTP_405
+
         if isinstance(fault, InvalidCredentialsError):
             return HTTP_401
+
         if isinstance(fault, Fault) and (fault.faultcode.startswith('Client.')
                                                 or fault.faultcode == 'Client'):
             return HTTP_400
@@ -193,46 +193,78 @@ class OutProtocolBase(ProtocolMixin):
 
         self.validator = None
 
-    def to_string(self, class_, value, *args, **kwargs):
+    def to_bytes(self, cls, value, *args, **kwargs):
         if value is None:
             return None
 
-        handler = self._to_string_handlers[class_]
-        retval = handler(class_, value, *args, **kwargs)
-        if six.PY3 and isinstance(retval, six.text_type):
-            retval = retval.encode('ascii')
+        handler = self._to_bytes_handlers[cls]
+        retval = handler(cls, value, *args, **kwargs)
+
+        # enable this only for testing. we're not as strict for performance
+        # reasons
+        # assert isinstance(retval, six.binary_type), \
+        #                 "AssertionError: %r %r %r handler: %r" % \
+        #                       (type(retval), six.binary_type, retval, handler)
         return retval
 
-    def to_unicode(self, class_, value, *args, **kwargs):
+    def to_unicode(self, cls, value, *args, **kwargs):
         if value is None:
             return None
 
-        handler = self._to_unicode_handlers[class_]
-        return handler(class_, value, *args, **kwargs)
+        handler = self._to_unicode_handlers[cls]
+        retval = handler(cls, value, *args, **kwargs)
 
-    def to_string_iterable(self, class_, value):
+        # enable this only for testing. we're not as strict for performance
+        # reasons as well as not to take the joy of dealing with duck typing
+        # from the user
+        # assert isinstance(retval, six.text_type), \
+        #                  "AssertionError: %r %r handler: %r" % \
+        #                                        (type(retval), retval, handler)
+
+        return retval
+
+    def to_bytes_iterable(self, cls, value):
         if value is None:
             return []
 
-        handler = self._to_string_iterable_handlers[class_]
-        return handler(class_, value)
+        if isinstance(value, PushBase):
+            return value
 
-    def null_to_string(self, cls, value, **_):
-        return ""
+        handler = self._to_bytes_iterable_handlers[cls]
+        return handler(cls, value)
 
-    def any_xml_to_string(self, cls, value, **_):
+    def null_to_bytes(self, cls, value, **_):
+        return b""
+
+    def null_to_unicode(self, cls, value, **_):
+        return u""
+
+    def any_xml_to_bytes(self, cls, value, **_):
         return etree.tostring(value)
 
     def any_xml_to_unicode(self, cls, value, **_):
         return etree.tostring(value, encoding='unicode')
 
-    def any_html_to_string(self, cls, value, **_):
+    def any_dict_to_unicode(self, cls, value, **_):
+        return repr(value)
+
+    def any_html_to_bytes(self, cls, value, **_):
         return html.tostring(value)
 
     def any_html_to_unicode(self, cls, value, **_):
         return html.tostring(value, encoding='unicode')
 
-    def uuid_to_string(self, cls, value, suggested_encoding=None, **_):
+    def uuid_to_bytes(self, cls, value, suggested_encoding=None, **_):
+        ser_as = self.get_cls_attrs(cls).serialize_as
+        retval = self.uuid_to_unicode(cls, value,
+                                     suggested_encoding=suggested_encoding, **_)
+
+        if ser_as in ('bytes', 'bytes_le', 'fields', 'int', six.binary_type):
+            return retval
+
+        return retval.encode('ascii')
+
+    def uuid_to_unicode(self, cls, value, suggested_encoding=None, **_):
         attr = self.get_cls_attrs(cls)
         ser_as = attr.serialize_as
         encoding = attr.encoding
@@ -245,13 +277,19 @@ class OutProtocolBase(ProtocolMixin):
             retval = binary_encoding_handlers[encoding]((retval,))
         return retval
 
-    def unicode_to_string(self, cls, value, **_):
+    def unicode_to_bytes(self, cls, value, **_):
         retval = value
 
         cls_attrs = self.get_cls_attrs(cls)
 
-        if cls_attrs.encoding is not None and isinstance(value, six.text_type):
-            retval = value.encode(cls_attrs.encoding)
+        if isinstance(value, six.text_type):
+            if cls_attrs.encoding is not None:
+                retval = value.encode(cls_attrs.encoding)
+            elif self.default_string_encoding is not None:
+                retval = value.encode(self.default_string_encoding)
+            elif not six.PY2:
+                logger.warning("You need to set either an encoding for %r "
+                               "or a default_string_encoding for %r", cls, self)
 
         if cls_attrs.str_format is not None:
             return cls_attrs.str_format.format(value)
@@ -260,20 +298,24 @@ class OutProtocolBase(ProtocolMixin):
 
         return retval
 
+    def any_uri_to_unicode(self, cls, value, **_):
+        return self.unicode_to_unicode(cls, value, **_)
+
     def unicode_to_unicode(self, cls, value, **_):  # :)))
-        # value can be many things, but definetly not an int
-        if isinstance(value, six.integer_types):
-            logger.warning("Returning an int where a str is expected! "
-                                                       "Silenty fixing this...")
-            value = str(value)
+        cls_attrs = self.get_cls_attrs(cls)
 
         retval = value
 
-        cls_attrs = self.get_cls_attrs(cls)
+        if isinstance(value, six.binary_type):
+            if cls_attrs.encoding is not None:
+                retval = value.decode(cls_attrs.encoding)
 
-        if cls_attrs.encoding is not None and \
-                                             isinstance(value, six.binary_type):
-            retval = value.decode(cls_attrs.encoding)
+            if self.default_string_encoding is not None:
+                retval = value.decode(self.default_string_encoding)
+
+            elif not six.PY2:
+                logger.warning("You need to set either an encoding for %r "
+                               "or a default_string_encoding for %r", cls, self)
 
         if cls_attrs.str_format is not None:
             return cls_attrs.str_format.format(value)
@@ -282,7 +324,10 @@ class OutProtocolBase(ProtocolMixin):
 
         return retval
 
-    def decimal_to_string(self, cls, value, **_):
+    def decimal_to_bytes(self, cls, value, **_):
+        return self.decimal_to_unicode(cls, value, **_).encode('utf8')
+
+    def decimal_to_unicode(self, cls, value, **_):
         D(value)  # sanity check
         cls_attrs = self.get_cls_attrs(cls)
 
@@ -293,7 +338,10 @@ class OutProtocolBase(ProtocolMixin):
 
         return str(value)
 
-    def double_to_string(self, cls, value, **_):
+    def double_to_bytes(self, cls, value, **_):
+        return self.double_to_unicode(cls, value, **_).encode('utf8')
+
+    def double_to_unicode(self, cls, value, **_):
         float(value) # sanity check
         cls_attrs = self.get_cls_attrs(cls)
 
@@ -304,7 +352,10 @@ class OutProtocolBase(ProtocolMixin):
 
         return repr(value)
 
-    def integer_to_string(self, cls, value, **_):
+    def integer_to_bytes(self, cls, value, **_):
+        return self.integer_to_unicode(cls, value, **_).encode('utf8')
+
+    def integer_to_unicode(self, cls, value, **_):
         int(value)  # sanity check
         cls_attrs = self.get_cls_attrs(cls)
 
@@ -315,29 +366,58 @@ class OutProtocolBase(ProtocolMixin):
 
         return str(value)
 
-    def time_to_string(self, cls, value, **_):
-        """Returns ISO formatted dates."""
+    def time_to_bytes(self, cls, value, **kwargs):
+        return self.time_to_unicode(cls, value, **kwargs)
+
+    def time_to_unicode(self, cls, value, **_):
+        """Returns ISO formatted times."""
+        if isinstance(value, datetime):
+            value = value.time()
         return value.isoformat()
 
-    def datetime_to_string(self, cls, val):
+    def date_to_bytes(self, cls, val, **_):
+        return self.date_to_unicode(cls, val, **_).encode("utf8")
+
+    def date_to_unicode(self, cls, val, **_):
+        if isinstance(val, datetime):
+            val = val.date()
+
         sa = self.get_cls_attrs(cls).serialize_as
 
-        if sa is None:
-            return self._datetime_to_string(cls, val)
+        if sa is None or sa in (str, 'str'):
+            return self._date_to_bytes(cls, val)
 
         return _datetime_smap[sa](cls, val)
 
-    def duration_to_string(self, cls, value, **_):
+    def datetime_to_bytes(self, cls, val, **_):
+        retval = self.datetime_to_unicode(cls, val, **_)
+        sa = self.get_cls_attrs(cls).serialize_as
+        if sa is None or sa in (six.text_type, str, 'str'):
+            return retval.encode('ascii')
+        return retval
+
+    def datetime_to_unicode(self, cls, val, **_):
+        sa = self.get_cls_attrs(cls).serialize_as
+
+        if sa is None or sa in (six.text_type, str, 'str'):
+            return self._datetime_to_unicode(cls, val)
+
+        return _datetime_smap[sa](cls, val)
+
+    def duration_to_bytes(self, cls, value, **_):
+        return self.duration_to_unicode(cls, value, **_).encode("utf8")
+
+    def duration_to_unicode(self, cls, value, **_):
         if value.days < 0:
             value = -value
             negative = True
         else:
             negative = False
 
-        tot_sec = _total_seconds(value)
+        tot_sec = int(value.total_seconds())
         seconds = value.seconds % 60
-        minutes = value.seconds / 60
-        hours = minutes / 60
+        minutes = value.seconds // 60
+        hours = minutes // 60
         minutes %= 60
         seconds = float(seconds)
         useconds = value.microseconds
@@ -348,9 +428,7 @@ class OutProtocolBase(ProtocolMixin):
         else:
             retval.append("P")
         if value.days != 0:
-            retval.extend([
-                "%iD" % value.days,
-                ])
+            retval.append("%iD" % value.days)
 
         if tot_sec != 0 and tot_sec % 86400 == 0 and useconds == 0:
             return ''.join(retval)
@@ -374,28 +452,57 @@ class OutProtocolBase(ProtocolMixin):
 
         return ''.join(retval)
 
-    def boolean_to_string(self, cls, value, **_):
+    def boolean_to_bytes(self, cls, value, **_):
+        return str(bool(value)).lower().encode('ascii')
+
+    def boolean_to_unicode(self, cls, value, **_):
         return str(bool(value)).lower()
 
-    def byte_array_to_string(self, cls, value, suggested_encoding=None, **_):
-        encoding = self.get_cls_attrs(cls).encoding
+    def byte_array_to_bytes(self, cls, value, suggested_encoding=None, **_):
+        cls_attrs = self.get_cls_attrs(cls)
+
+        encoding = cls_attrs.encoding
         if encoding is BINARY_ENCODING_USE_DEFAULT:
-            encoding = suggested_encoding
-        return binary_encoding_handlers[encoding](value)
+            if suggested_encoding is None:
+                encoding = self.binary_encoding
+            else:
+                encoding = suggested_encoding
+
+        if encoding is None and isinstance(value, (list, tuple)) \
+                             and len(value) == 1 and isinstance(value[0], mmap):
+            return value[0]
+
+        encoder = binary_encoding_handlers[encoding]
+        logger.debug("Using binary encoder %r for encoding %r",
+                                                              encoder, encoding)
+        retval = encoder(value)
+        if encoding is not None and isinstance(retval, six.text_type):
+            retval = retval.encode('ascii')
+
+        return retval
 
     def byte_array_to_unicode(self, cls, value, suggested_encoding=None, **_):
         encoding = self.get_cls_attrs(cls).encoding
         if encoding is BINARY_ENCODING_USE_DEFAULT:
-            encoding = suggested_encoding
+            if suggested_encoding is None:
+                encoding = self.binary_encoding
+            else:
+                encoding = suggested_encoding
+
         if encoding is None:
             raise ValueError("Arbitrary binary data can't be serialized to "
-                             "unicode")
-        return binary_encoding_handlers[encoding](value)
+                                                                      "unicode")
 
-    def byte_array_to_string_iterable(self, cls, value, **_):
+        retval = binary_encoding_handlers[encoding](value)
+        if not isinstance(retval, six.text_type):
+            retval = retval.decode('ascii')
+
+        return retval
+
+    def byte_array_to_bytes_iterable(self, cls, value, **_):
         return value
 
-    def file_to_string(self, cls, value, suggested_encoding=None):
+    def file_to_bytes(self, cls, value, suggested_encoding=None):
         """
         :param cls: A :class:`spyne.model.File` subclass
         :param value: Either a sequence of byte chunks or a
@@ -404,23 +511,51 @@ class OutProtocolBase(ProtocolMixin):
 
         encoding = self.get_cls_attrs(cls).encoding
         if encoding is BINARY_ENCODING_USE_DEFAULT:
-            encoding = suggested_encoding
+            if suggested_encoding is None:
+                encoding = self.binary_encoding
+            else:
+                encoding = suggested_encoding
 
         if isinstance(value, File.Value):
             if value.data is not None:
                 return binary_encoding_handlers[encoding](value.data)
 
             if value.handle is not None:
-                assert isinstance(value.handle, file)
+                # maybe we should have used the sweeping except: here.
+                if hasattr(value.handle, 'fileno'):
+                    if six.PY2:
+                        fileno = value.handle.fileno()
+                        data = (mmap(fileno, 0, access=ACCESS_READ),)
+                    else:
+                        import io
+                        try:
+                            fileno = value.handle.fileno()
+                            data = mmap(fileno, 0, access=ACCESS_READ)
+                        except io.UnsupportedOperation:
+                            data = (value.handle.read(),)
+                else:
+                    data = (value.handle.read(),)
 
-                fileno = value.handle.fileno()
+                return binary_encoding_handlers[encoding](data)
+
+            if value.path is not None:
+                handle = open(value.path, 'rb')
+                fileno = handle.fileno()
                 data = mmap(fileno, 0, access=ACCESS_READ)
 
                 return binary_encoding_handlers[encoding](data)
 
-            assert False
+            assert False, "Unhandled file type"
 
-        return binary_encoding_handlers[encoding](value)
+        if value is None:
+            return b''
+
+        try:
+            return binary_encoding_handlers[encoding](value)
+        except Exception as e:
+            logger.error("Error encoding value to binary. Error: %r, Value: %r",
+                                                                       e, value)
+            raise
 
     def file_to_unicode(self, cls, value, suggested_encoding=None):
         """
@@ -434,19 +569,21 @@ class OutProtocolBase(ProtocolMixin):
         if encoding is BINARY_ENCODING_USE_DEFAULT:
             encoding = suggested_encoding
 
-        if encoding is None and cls_attrs.type is File.BINARY:
+        if encoding is None and cls_attrs.mode is File.TEXT:
             raise ValueError("Arbitrary binary data can't be serialized to "
                              "unicode.")
 
-        return self.file_to_string(cls, value, suggested_encoding)
+        retval = self.file_to_bytes(cls, value, suggested_encoding)
+        if not isinstance(retval, six.text_type):
+            retval = retval.decode('ascii')
+        return retval
 
-    def file_to_string_iterable(self, cls, value, **_):
+    def file_to_bytes_iterable(self, cls, value, **_):
         if value.data is not None:
             if isinstance(value.data, (list, tuple)) and \
-                    isinstance(value.data[0], mmap):
+                                                isinstance(value.data[0], mmap):
                 return _file_to_iter(value.data[0])
-            else:
-                return iter(value.data)
+            return iter(value.data)
 
         if value.handle is not None:
             f = value.handle
@@ -460,6 +597,8 @@ class OutProtocolBase(ProtocolMixin):
             path = value.path
             if not isabs(value.path):
                 path = join(value.store, value.path)
+                assert abspath(path).startswith(value.store), \
+                                                 "No relative paths are allowed"
             return _file_to_iter(open(path, 'rb'))
 
         except IOError as e:
@@ -468,122 +607,208 @@ class OutProtocolBase(ProtocolMixin):
             else:
                 raise InternalError("Error accessing requested file")
 
-    def simple_model_to_string_iterable(self, cls, value, **kwargs):
-        retval = self.to_string(cls, value, **kwargs)
+    def simple_model_to_bytes_iterable(self, cls, value, **kwargs):
+        retval = self.to_bytes(cls, value, **kwargs)
         if retval is None:
-            return ('',)
+            return (b'',)
         return (retval,)
 
-    def complex_model_to_string_iterable(self, cls, value, **_):
+    def complex_model_to_bytes_iterable(self, cls, value, **_):
         if self.ignore_uncap:
             return tuple()
         raise TypeError("This protocol can only serialize primitives.")
 
-    def attachment_to_string(self, cls, value, **_):
-        if not (value.data is None):
-            # the data has already been loaded, just encode
-            # and return the element
-            data = value.data
-
-        elif not (value.file_name is None):
-            # the data hasn't been loaded, but a file has been
-            # specified
-            data = open(value.file_name, 'rb').read()
-
-        else:
-            raise ValueError("Neither data nor a file_name has been specified")
-
-        return data
-
-    def complex_model_base_to_string(self, cls, value, **_):
+    def complex_model_base_to_bytes(self, cls, value, **_):
         raise TypeError("Only primitives can be serialized to string.")
 
-    def xmlattribute_to_string(self, cls, string, **kwargs):
-        return self.to_string(cls.type, string, **kwargs)
+    def complex_model_base_to_unicode(self, cls, value, **_):
+        raise TypeError("Only primitives can be serialized to string.")
+
+    def xmlattribute_to_bytes(self, cls, string, **kwargs):
+        return self.to_bytes(cls.type, string, **kwargs)
 
     def xmlattribute_to_unicode(self, cls, string, **kwargs):
         return self.to_unicode(cls.type, string, **kwargs)
 
-    def model_base_to_string_iterable(self, cls, value, **kwargs):
-        return cls.to_string_iterable(value, **kwargs)
+    def model_base_to_bytes_iterable(self, cls, value, **kwargs):
+        return cls.to_bytes_iterable(value, **kwargs)
 
-    def model_base_to_string(self, cls, value, **kwargs):
-        return cls.to_string(value, **kwargs)
+    def model_base_to_bytes(self, cls, value, **kwargs):
+        return cls.to_bytes(value, **kwargs)
 
     def model_base_to_unicode(self, cls, value, **kwargs):
         return cls.to_unicode(value, **kwargs)
 
-    def _datetime_to_string(self, cls, value, **_):
+    def _datetime_to_unicode(self, cls, value, **_):
+        """Returns ISO formatted datetimes."""
+
         cls_attrs = self.get_cls_attrs(cls)
+
         if cls_attrs.as_timezone is not None and value.tzinfo is not None:
             value = value.astimezone(cls_attrs.as_timezone)
+
         if not cls_attrs.timezone:
             value = value.replace(tzinfo=None)
 
-        # FIXME: this should be date_format, all other aliases are to be
-        # deprecated
-        date_format = cls_attrs.date_format
-        if date_format is None:
-            date_format = cls_attrs.out_format
-        if date_format is None:
-            date_format = cls_attrs.format
+        dt_format = self._get_datetime_format(cls_attrs)
 
-        if date_format is None:
+        if dt_format is None:
             retval = value.isoformat()
-        elif six.PY2 and isinstance(date_format, unicode):
-            retval = value.strftime(date_format.encode('utf8')).decode('utf8')
+
+        elif six.PY2 and isinstance(dt_format, unicode):
+            retval = self.strftime(value, dt_format.encode('utf8')).decode('utf8')
+
         else:
-            retval = value.strftime(date_format)
+            retval = self.strftime(value, dt_format)
 
         # FIXME: must deprecate string_format, this should have been str_format
-        string_format = cls_attrs.string_format
-        if string_format is None:
-            string_format = cls_attrs.str_format
-        if string_format is not None:
-            return string_format.format(value)
+        str_format = cls_attrs.string_format
+        if str_format is None:
+            str_format = cls_attrs.str_format
+        if str_format is not None:
+            return str_format.format(value)
 
         # FIXME: must deprecate interp_format, this should have been just format
-        format = cls_attrs.interp_format
+        interp_format = cls_attrs.interp_format
+        if interp_format is not None:
+            return interp_format.format(value)
+
+        return retval
+
+    def _date_to_bytes(self, cls, value, **_):
+        cls_attrs = self.get_cls_attrs(cls)
+
+        date_format = cls_attrs.date_format
+        if date_format is None:
+            retval = value.isoformat()
+
+        elif six.PY2 and isinstance(date_format, unicode):
+            date_format = date_format.encode('utf8')
+            retval = self.strftime(value, date_format).decode('utf8')
+
+        else:
+            retval = self.strftime(value, date_format)
+
+        str_format = cls_attrs.str_format
+        if str_format is not None:
+            return str_format.format(value)
+
+        format = cls_attrs.format
         if format is not None:
             return format.format(value)
 
         return retval
 
+    # Format a datetime through its full proleptic Gregorian date range.
+    # http://code.activestate.com/recipes/
+    #                306860-proleptic-gregorian-dates-and-strftime-before-1900/
+    # http://stackoverflow.com/a/32206673
+    #
+    # >>> strftime(datetime.date(1850, 8, 2), "%Y/%M/%d was a %A")
+    # '1850/00/02 was a Friday'
+    # >>>
+
+
+    # remove the unsupposed "%s" command.  But don't
+    # do it if there's an even number of %s before the s
+    # because those are all escaped.  Can't simply
+    # remove the s because the result of
+    #  %sY
+    # should be %Y if %s isn't supported, not the
+    # 4 digit year.
+    _illegal_s = re.compile(r"((^|[^%])(%%)*%s)")
+
+    @staticmethod
+    def _findall_datetime(text, substr):
+         # Also finds overlaps
+         sites = []
+         i = 0
+         while 1:
+             j = text.find(substr, i)
+             if j == -1:
+                 break
+             sites.append(j)
+             i=j+1
+         return sites
+
+    # Every 28 years the calendar repeats, except through century leap
+    # years where it's 6 years.  But only if you're using the Gregorian
+    # calendar.  ;)
+
+    @classmethod
+    def strftime(cls, dt, fmt):
+        if cls._illegal_s.search(fmt):
+            raise TypeError("This strftime implementation does not handle %s")
+        if dt.year > 1900:
+            return dt.strftime(fmt)
+
+        year = dt.year
+        # For every non-leap year century, advance by
+        # 6 years to get into the 28-year repeat cycle
+        delta = 2000 - year
+        off = 6*(delta // 100 + delta // 400)
+        year += off
+
+        # Move to around the year 2000
+        year += ((2000 - year) // 28) * 28
+        timetuple = dt.timetuple()
+        s1 = strftime(fmt, (year,) + timetuple[1:])
+        sites1 = cls._findall_datetime(s1, str(year))
+
+        s2 = strftime(fmt, (year+28,) + timetuple[1:])
+        sites2 = cls._findall_datetime(s2, str(year+28))
+
+        sites = []
+        for site in sites1:
+            if site in sites2:
+                sites.append(site)
+
+        s = s1
+        syear = "%4d" % (dt.year,)
+        for site in sites:
+            s = s[:site] + syear + s[site+4:]
+        return s
+
 
 _uuid_serialize = {
     None: str,
+    str: str,
+    'str': str,
+
     'hex': lambda u: u.hex,
     'urn': lambda u: u.urn,
     'bytes': lambda u: u.bytes,
     'bytes_le': lambda u: u.bytes_le,
     'fields': lambda u: u.fields,
+
+    int: lambda u: u.int,
     'int': lambda u: u.int,
 }
 
 _uuid_deserialize = {
-    None: lambda s: uuid.UUID(s),
+    None: uuid.UUID,
+    str: uuid.UUID,
+    'str': uuid.UUID,
+
     'hex': lambda s: uuid.UUID(hex=s),
     'urn': lambda s: uuid.UUID(hex=s),
     'bytes': lambda s: uuid.UUID(bytes=s),
     'bytes_le': lambda s: uuid.UUID(bytes_le=s),
     'fields': lambda s: uuid.UUID(fields=s),
+
+    int: lambda s: uuid.UUID(int=s),
     'int': lambda s: uuid.UUID(int=s),
+
+    (int, int): lambda s: uuid.UUID(int=s),
     ('int', int): lambda s: uuid.UUID(int=s),
+
+    (int, str): lambda s: uuid.UUID(int=int(s)),
     ('int', str): lambda s: uuid.UUID(int=int(s)),
 }
 
 if six.PY2:
     _uuid_deserialize[('int', long)] = _uuid_deserialize[('int', int)]
-
-
-
-if hasattr(timedelta, 'total_seconds'):
-    def _total_seconds(td):
-        return td.total_seconds()
-
-else:
-    def _total_seconds(td):
-        return (td.microseconds + (td.seconds + td.days * 24 * 3600) *1e6) / 1e6
+    _uuid_deserialize[(int, long)] = _uuid_deserialize[('int', int)]
 
 
 def _parse_datetime_iso_match(date_match, tz=None):
@@ -655,10 +880,10 @@ _datetime_smap = {
 
 def _file_to_iter(f):
     try:
-        data = f.read(65536)
+        data = f.read(8192)
         while len(data) > 0:
             yield data
-            data = f.read(65536)
+            data = f.read(8192)
 
     finally:
         f.close()
