@@ -29,30 +29,41 @@ from __future__ import absolute_import
 import logging
 logger = logging.getLogger(__name__)
 
-from spyne.util import six
-
 import msgpack
 
+from spyne import ValidationError
+from spyne.util import six
 from spyne.model.fault import Fault
-from spyne.protocol.dictdoc import HierDictDocument
 from spyne.model.primitive import Double
 from spyne.model.primitive import Boolean
 from spyne.model.primitive import Integer
+from spyne.protocol.dictdoc import HierDictDocument
 
 
 class MessagePackDecodeError(Fault):
+    CODE = "Client.MessagePackDecodeError"
+
     def __init__(self, data=None):
-        super(MessagePackDecodeError, self).__init__("Client.MessagePackDecodeError", data)
+        super(MessagePackDecodeError, self) \
+                                .__init__(self.CODE, data)
+
+
+NON_NUMBER_TYPES = tuple({list, dict, six.text_type, six.binary_type})
 
 
 class MessagePackDocument(HierDictDocument):
     """An integration class for the msgpack protocol."""
 
     mime_type = 'application/x-msgpack'
+    text_based = False
 
     type = set(HierDictDocument.type)
     type.add('msgpack')
 
+    default_string_encoding = 'UTF-8'
+    from_serstr = HierDictDocument.from_bytes
+    to_serstr = HierDictDocument.to_bytes
+
     # flags to be used in tests
     _decimal_as_string = True
     _huge_numbers_as_string = True
@@ -63,22 +74,50 @@ class MessagePackDocument(HierDictDocument):
                                         ignore_wrappers=True,
                                         complex_as=dict,
                                         ordered=False,
-                                        polymorphic=False):
+                                        polymorphic=False,
+                                        key_encoding='utf8',
+                                        # MessagePackDocument specific
+                                        use_list=False,
+                                        mw_unpacker=msgpack.Unpacker):
 
         super(MessagePackDocument, self).__init__(app, validator, mime_type,
-                ignore_uncap, ignore_wrappers, complex_as, ordered, polymorphic)
+                ignore_uncap, ignore_wrappers, complex_as, ordered, polymorphic,
+                                                                   key_encoding)
 
-        self._from_unicode_handlers[Double] = self._ret
-        self._from_unicode_handlers[Boolean] = self._ret
-        self._from_unicode_handlers[Integer] = self.integer_from_string
+        self.use_list = use_list
+        self.mw_unpacker = mw_unpacker
 
-        self._to_unicode_handlers[Double] = self._ret
-        self._to_unicode_handlers[Boolean] = self._ret
-        self._to_unicode_handlers[Integer] = self.integer_to_string
+        self._from_bytes_handlers[Double] = self._ret_number
+        self._from_bytes_handlers[Boolean] = self._ret_bool
+        self._from_bytes_handlers[Integer] = self.integer_from_bytes
 
-    def _ret(self, cls, value):
+        self._to_bytes_handlers[Double] = self._ret_number
+        self._to_bytes_handlers[Boolean] = self._ret_bool
+        self._to_bytes_handlers[Integer] = self.integer_to_bytes
+
+    def _ret(self, _, value):
         return value
 
+    def _ret_number(self, _, value):
+        if isinstance(value, NON_NUMBER_TYPES):
+            raise ValidationError(value)
+        if value in (True, False):
+            return int(value)
+        return value
+
+    def _ret_bool(self, _, value):
+        if value is None or value in (True, False):
+            return value
+        raise ValidationError(value)
+
+    def get_class_name(self, cls):
+        class_name = cls.get_type_name()
+        if not six.PY2:
+            if not isinstance(class_name, bytes):
+                class_name = class_name.encode('utf8')
+
+        return class_name
+
     def create_in_document(self, ctx, in_string_encoding=None):
         """Sets ``ctx.in_document``,  using ``ctx.in_string``.
 
@@ -87,33 +126,48 @@ class MessagePackDocument(HierDictDocument):
             argument is ignored.
         """
 
-        # TODO: Use feed api as msgpack's implementation reads everything in one
-        # go anyway.
-
         # handle mmap objects from in ctx.in_string as returned by
         # TwistedWebResource.handle_rpc.
-        in_string = ((s.read(s.size()) if hasattr(s, 'read') else s)
-                                                         for s in ctx.in_string)
-        try:
-            ctx.in_document = msgpack.unpackb(b''.join(in_string))
-        except ValueError as e:
-            raise MessagePackDecodeError(''.join(e.args))
+        if isinstance(ctx.in_string, (list, tuple)) \
+                               and len(ctx.in_string) == 1 \
+                               and isinstance(ctx.in_string[0], memoryview):
+            unpacker = self.mw_unpacker(use_list=self.use_list)
+            unpacker.feed(ctx.in_string[0])
+            ctx.in_document = next(x for x in unpacker)
+
+        else:
+            try:
+                ctx.in_document = msgpack.unpackb(b''.join(ctx.in_string))
+            except ValueError as e:
+                raise MessagePackDecodeError(' '.join(e.args))
+
+    def gen_method_request_string(self, ctx):
+        """Uses information in context object to return a method_request_string.
+
+        Returns a string in the form of "{namespaces}method name".
+        """
+
+        mrs, = ctx.in_body_doc.keys()
+        if not six.PY2:
+            mrs = mrs.decode('utf8')
+
+        return '{%s}%s' % (self.app.interface.get_tns(), mrs)
 
     def create_out_string(self, ctx, out_string_encoding='utf8'):
         ctx.out_string = (msgpack.packb(o) for o in ctx.out_document)
 
-    def integer_from_string(self, cls, value):
-        if isinstance(value, six.string_types):
-            return super(MessagePackDocument, self).integer_from_string(cls, value)
-        else:
-            return value
+    def integer_from_bytes(self, cls, value):
+        if isinstance(value, (six.text_type, six.binary_type)):
+            return super(MessagePackDocument, self) \
+                                                .integer_from_bytes(cls, value)
+        return value
 
-    def integer_to_string(self, cls, value):
+    def integer_to_bytes(self, cls, value, **_):
         # if it's inside the range msgpack can deal with
         if -1<<63 <= value < 1<<64:
             return value
         else:
-            return super(MessagePackDocument, self).integer_to_string(cls, value)
+            return super(MessagePackDocument, self).integer_to_bytes(cls, value)
 
 
 class MessagePackRpc(MessagePackDocument):
@@ -124,6 +178,7 @@ class MessagePackRpc(MessagePackDocument):
     MSGPACK_REQUEST = 0
     MSGPACK_RESPONSE = 1
     MSGPACK_NOTIFY = 2
+    MSGPACK_ERROR = 3
 
     def create_out_string(self, ctx, out_string_encoding='utf8'):
         ctx.out_string = (msgpack.packb(o) for o in ctx.out_document)
@@ -138,7 +193,8 @@ class MessagePackRpc(MessagePackDocument):
 
         # TODO: Use feed api
         try:
-            ctx.in_document = msgpack.unpackb(b''.join(ctx.in_string))
+            ctx.in_document = msgpack.unpackb(b''.join(ctx.in_string),
+                                                         use_list=self.use_list)
         except ValueError as e:
             raise MessagePackDecodeError(''.join(e.args))
 
@@ -163,6 +219,11 @@ class MessagePackRpc(MessagePackDocument):
         else:
             msgtype, msgid, msgname_or_error, msgparams = ctx.in_document
 
+        if not six.PY2:
+            if isinstance(msgname_or_error, bytes):
+                msgname_or_error = msgname_or_error.decode(
+                                                   self.default_string_encoding)
+
         if msgtype == MessagePackRpc.MSGPACK_REQUEST:
             assert message == MessagePackRpc.REQUEST
 
@@ -176,9 +237,10 @@ class MessagePackRpc(MessagePackDocument):
             raise MessagePackDecodeError("Unknown message type %r" % msgtype)
 
         ctx.method_request_string = '{%s}%s' % (self.app.interface.get_tns(),
-                                                                        msgname_or_error)
+                                                               msgname_or_error)
 
-        ctx.in_header_doc = None # MessagePackRpc does not seem to have Header support
+        # MessagePackRpc does not seem to have Header support
+        ctx.in_header_doc = None
 
         if isinstance(msgname_or_error, dict) and msgname_or_error:
             # we got an error
@@ -186,8 +248,8 @@ class MessagePackRpc(MessagePackDocument):
         else:
             ctx.in_body_doc = msgparams
 
-        logger.debug('\theader : %r', ctx.in_header_doc)
-        logger.debug('\tbody   : %r', ctx.in_body_doc)
+        # logger.debug('\theader : %r', ctx.in_header_doc)
+        # logger.debug('\tbody   : %r', ctx.in_body_doc)
 
     def deserialize(self, ctx, message):
         assert message in (self.REQUEST, self.RESPONSE)
@@ -210,7 +272,8 @@ class MessagePackRpc(MessagePackDocument):
             ctx.in_error = Fault(**ctx.in_error)
 
         elif body_class:
-            ctx.in_object = self._doc_to_object(body_class, ctx.in_body_doc, self.validator)
+            ctx.in_object = self._doc_to_object(ctx,
+                                    body_class, ctx.in_body_doc, self.validator)
 
         else:
             ctx.in_object = []
@@ -224,7 +287,8 @@ class MessagePackRpc(MessagePackDocument):
 
         if ctx.out_error is not None:
             ctx.out_document = [
-                [MessagePackRpc.MSGPACK_RESPONSE, 0, Fault.to_dict(ctx.out_error.__class__, ctx.out_error)]
+                [MessagePackRpc.MSGPACK_ERROR, 0,
+                          Fault.to_dict(ctx.out_error.__class__, ctx.out_error)]
             ]
             return
 
@@ -252,14 +316,15 @@ class MessagePackRpc(MessagePackDocument):
 
         # assign raw result to its wrapper, result_message
         for i, (k, v) in enumerate(out_type_info.items()):
-            attr_name = k
-            out_instance._safe_set(attr_name, ctx.out_object[i], v)
+            attrs = self.get_cls_attrs(v)
+            out_instance._safe_set(k, ctx.out_object[i], v, attrs)
 
         # transform the results into a dict:
         if out_type.Attributes.max_occurs > 1:
-            params = (self._to_dict_value(out_type, inst) for inst in out_instance)
+            params = (self._to_dict_value(out_type, inst, set())
+                                                       for inst in out_instance)
         else:
-            params = self._to_dict_value(out_type, out_instance)
+            params = self._to_dict_value(out_type, out_instance, set())
 
         ctx.out_document = [[msgtype, 0, method_name_or_error, params]]
 
