@@ -35,50 +35,62 @@ memory for later requests. However, if you want to make sure you only have this
 url on the WSDL, this is how to do it. Note that if your client takes the
 information in wsdl seriously, all requests will go to the designated url above
 which can make testing a bit difficult. Use in moderation.
-
-This module is EXPERIMENTAL. Your mileage may vary. Patches are welcome.
 """
 
-
 from __future__ import absolute_import
 
 import logging
 logger = logging.getLogger(__name__)
 
 import re
+import cgi
+import threading
 
 from os import fstat
 from mmap import mmap
-from inspect import isgenerator, isclass
+from inspect import isclass
 from collections import namedtuple
 
+from twisted.web import static
 from twisted.web.server import NOT_DONE_YET, Request
 from twisted.web.resource import Resource, NoResource, ForbiddenResource
-from twisted.web import static
 from twisted.web.static import getTypeAndEncoding
-from twisted.web.http import CACHED
 from twisted.python.log import err
+from twisted.python.failure import Failure
+from twisted.internet import reactor
+from twisted.internet.task import deferLater
 from twisted.internet.defer import Deferred
+from twisted.internet.threads import deferToThread
 
-from spyne import BODY_STYLE_BARE, BODY_STYLE_EMPTY, Redirect
+from spyne import Redirect, Address
 from spyne.application import logger_server
 from spyne.application import get_fault_string_from_exception
 
+from spyne.util import six
 from spyne.error import InternalError
 from spyne.auxproc import process_contexts
 from spyne.const.ansi_color import LIGHT_GREEN
 from spyne.const.ansi_color import END_COLOR
 from spyne.const.http import HTTP_404, HTTP_200
+
 from spyne.model import PushBase, File, ComplexModelBase
 from spyne.model.fault import Fault
+
 from spyne.protocol.http import HttpRpc
+
 from spyne.server.http import HttpBase
 from spyne.server.http import HttpMethodContext
 from spyne.server.http import HttpTransportContext
 from spyne.server.twisted._base import Producer
+from spyne.server.twisted import log_and_let_go
+
+from spyne.util.address import address_parser
 from spyne.util.six import text_type, string_types
 from spyne.util.six.moves.urllib.parse import unquote
 
+if not six.PY2:
+    from urllib.request import unquote_to_bytes
+
 
 def _render_file(file, request):
     """
@@ -137,7 +149,8 @@ def _set_response_headers(request, headers):
 
 
 def _reconstruct_url(request):
-    server_name = request.getRequestHostname()
+    # HTTP "Hosts" header only supports ascii
+    server_name = request.getRequestHostname().decode('ascii')
     server_port = request.getHost().port
     if (bool(request.isSecure()), server_port) not in [(True, 443), (False, 80)]:
         server_name = '%s:%d' % (server_name, server_port)
@@ -146,8 +159,23 @@ def _reconstruct_url(request):
         url_scheme = 'https'
     else:
         url_scheme = 'http'
+    uri = _decode_path(request.uri)
+    return ''.join([url_scheme, "://", server_name, uri])
 
-    return ''.join([url_scheme, "://", server_name, request.uri])
+
+class _Transformer(object):
+    def __init__(self, req):
+        self.req = req
+
+    def get(self, key, default):
+        key = key.lower()
+        if key.startswith((b'http_', b'http-')):
+            key = key[5:]
+
+        retval = self.req.getHeader(key)
+        if retval is None:
+            retval = default
+        return retval
 
 
 class TwistedHttpTransportContext(HttpTransportContext):
@@ -172,9 +200,29 @@ class TwistedHttpTransportContext(HttpTransportContext):
     def get_request_content_type(self):
         return self.req.getHeader("Content-Type")
 
+    def get_peer(self):
+        peer = Address.from_twisted_address(self.req.transport.getPeer())
+        addr = address_parser.get_ip(_Transformer(self.req))
+
+        if addr is None:
+            return peer
+
+        if address_parser.is_valid_ipv4(addr):
+            return Address(type=Address.TCP4, host=addr, port=0)
+
+        if address_parser.is_valid_ipv6(addr):
+            return Address(type=Address.TCP6, host=addr, port=0)
+
 
 class TwistedHttpMethodContext(HttpMethodContext):
-    default_transport_context = TwistedHttpTransportContext
+    HttpTransportContext = TwistedHttpTransportContext
+
+
+def _decode_path(fragment):
+    if six.PY2:
+        return unquote(fragment).decode('utf8')
+
+    return unquote_to_bytes(fragment).decode('utf8')
 
 
 class TwistedHttpTransport(HttpBase):
@@ -183,6 +231,79 @@ class TwistedHttpTransport(HttpBase):
         super(TwistedHttpTransport, self).__init__(app, chunked=chunked,
                max_content_length=max_content_length, block_length=block_length)
 
+        self.reactor_thread = None
+        def _cb():
+            self.reactor_thread = threading.current_thread()
+
+        deferLater(reactor, 0, _cb)
+
+    def pusher_init(self, p_ctx, gen, _cb_push_finish, pusher, interim):
+        if pusher.orig_thread != self.reactor_thread:
+            return deferToThread(super(TwistedHttpTransport, self).pusher_init,
+                                   p_ctx, gen, _cb_push_finish, pusher, interim)
+
+        return super(TwistedHttpTransport, self).pusher_init(
+                                   p_ctx, gen, _cb_push_finish, pusher, interim)
+
+    @staticmethod
+    def set_out_document_push(ctx):
+        class _ISwearImAGenerator(object):
+            def send(self, data):
+                if not data: return
+                ctx.out_stream.write(data)
+
+        ctx.out_document = _ISwearImAGenerator()
+
+    def pusher_try_close(self, ctx, pusher, retval):
+        # the whole point of this function is to call ctx.out_stream.finish()
+        # when a *root* pusher has no more data to send. interim pushers don't
+        # have to close anything.
+        if isinstance(retval, Deferred):
+            def _eb_push_close(f):
+                assert isinstance(f, Failure)
+
+                logger.error(f.getTraceback())
+
+                subretval = super(TwistedHttpTransport, self) \
+                                          .pusher_try_close(ctx, pusher, retval)
+
+                if not pusher.interim:
+                    ctx.out_stream.finish()
+
+                return subretval
+
+            def _cb_push_close(r):
+                def _eb_inner(f):
+                    if not pusher.interim:
+                        ctx.out_stream.finish()
+
+                    return f
+
+                if not isinstance(r, Deferred):
+                    retval = super(TwistedHttpTransport, self) \
+                                               .pusher_try_close(ctx, pusher, r)
+                    if not pusher.interim:
+                        ctx.out_stream.finish()
+
+                    return retval
+
+                return r \
+                    .addCallback(_cb_push_close) \
+                    .addErrback(_eb_inner) \
+                    .addErrback(log_and_let_go, logger)
+
+            return retval \
+                .addCallback(_cb_push_close) \
+                .addErrback(_eb_push_close)  \
+                .addErrback(log_and_let_go, logger)
+
+        super(TwistedHttpTransport, self).pusher_try_close(ctx, pusher, retval)
+
+        if not pusher.interim:
+            retval = ctx.out_stream.finish()
+
+        return retval
+
     def decompose_incoming_envelope(self, prot, ctx, message):
         """This function is only called by the HttpRpc protocol to have the
         twisted web's Request object is parsed into ``ctx.in_body_doc`` and
@@ -193,17 +314,17 @@ class TwistedHttpTransport(HttpBase):
         assert isinstance(request, Request)
 
         ctx.in_header_doc = dict(request.requestHeaders.getAllRawHeaders())
-        fi = ctx.transport.file_info
-        if fi is not None and len(request.args) == 1:
-            key, = request.args.keys()
-            if fi.field_name == key and fi.file_name is not None:
-                ctx.in_body_doc = {key: [File.Value(name=fi.file_name,
-                                    type=fi.file_type, data=request.args[key])]}
+        ctx.in_body_doc = request.args
 
-            else:
-                ctx.in_body_doc = request.args
-        else:
-            ctx.in_body_doc = request.args
+        for fi in ctx.transport.file_info:
+            assert isinstance(fi, _FileInfo)
+            data = request.args.get(fi.field_name, None)
+            if data is not None and fi.file_name is not None:
+                ctx.in_body_doc[fi.field_name] = \
+                    [File.Value(
+                        name=fi.file_name,
+                        type=fi.file_type,
+                        data=fi.data)]
 
         # this is a huge hack because twisted seems to take the slashes in urls
         # too seriously.
@@ -211,14 +332,18 @@ class TwistedHttpTransport(HttpBase):
         if postpath is None:
             postpath = request.path
 
+        if postpath is not None:
+            postpath = _decode_path(postpath)
+
         params = self.match_pattern(ctx, request.method, postpath,
-                                                      request.getHeader('Host'))
+                                                     request.getHeader(b'Host'))
 
         if ctx.method_request_string is None: # no pattern match
-            ctx.method_request_string = '{%s}%s' % (self.app.interface.get_tns(),
-                                                request.path.rsplit('/', 1)[-1])
+            ctx.method_request_string = u'{%s}%s' % (
+                                 self.app.interface.get_tns(),
+                                 _decode_path(request.path.rsplit(b'/', 1)[-1]))
 
-        logger.debug("%sMethod name: %r%s" % (LIGHT_GREEN,
+        logger.debug(u"%sMethod name: %r%s" % (LIGHT_GREEN,
                                           ctx.method_request_string, END_COLOR))
 
         for k, v in params.items():
@@ -246,58 +371,87 @@ class TwistedHttpTransport(HttpBase):
 
 FIELD_NAME_RE = re.compile(r'name="([^"]+)"')
 FILE_NAME_RE = re.compile(r'filename="([^"]+)"')
-_FileInfo = namedtuple("_FileInfo", "field_name file_name file_type "
-                                                                "header_offset")
-def _get_file_name(instr):
-    """We need this huge hack because twisted doesn't offer a way to get file
-    name from Content-Disposition header. This works only when there's just one
-    file because we want to avoid scanning the whole stream. So this won't get
-    the names of the subsequent files even though that's a perfectly valid
-    request.
+_FileInfo = namedtuple("_FileInfo", "field_name file_name file_type data")
+
+
+def _get_file_info(ctx):
+    """We need this hack because twisted doesn't offer a way to get file name
+    from Content-Disposition header.
     """
 
-    field_name = file_name = file_type = content_idx = None
+    retval = []
 
-    # hack to see if it looks like a multipart request. 5 is arbitrary.
-    if instr[:5] == "-----":
-        first_page = instr[:4096] # 4096 = default page size on linux.
+    request = ctx.transport.req
+    headers = request.getAllHeaders()
+    content_type = headers.get('content-type', None)
+    if content_type is None:
+        return retval
 
-        # this normally roughly <200
-        header_idx = first_page.find('\r\n') + 2
-        content_idx = first_page.find('\r\n\r\n', header_idx)
-        if header_idx > 0 and content_idx > header_idx:
-            headerstr = first_page[header_idx:content_idx]
+    img = cgi.FieldStorage(
+        fp=request.content,
+        headers=ctx.in_header_doc,
+        environ={
+            'REQUEST_METHOD': request.method,
+            'CONTENT_TYPE': content_type,
+        }
+    )
 
-            for line in headerstr.split("\r\n"):
-                k, v = line.split(":", 2)
-                if k == "Content-Disposition":
-                    for subv in v.split(";"):
-                        subv = subv.strip()
-                        m = FIELD_NAME_RE.match(subv)
-                        if m:
-                            field_name = m.group(1)
-                            continue
+    try:
+        keys = img.keys()
+    except TypeError:
+        return retval
 
-                        m = FILE_NAME_RE.match(subv)
-                        if m:
-                            file_name = m.group(1)
-                    continue
+    for k in keys:
+        fields = img[k]
 
-                if k == "Content-Type":
-                    file_type = v.strip()
+        if isinstance(fields, cgi.FieldStorage):
+            fields = (fields,)
 
-        # 4 == len('\r\n\r\n')
-        return _FileInfo(field_name, file_name, file_type, content_idx + 4)
+        for field in fields:
+            file_type = field.type
+            file_name = field.disposition_options.get('filename', None)
+            if file_name is not None:
+                retval.append(_FileInfo(k, file_name, file_type,
+                                                [mmap(field.file.fileno(), 0)]))
+
+    return retval
 
 
 def _has_fd(istr):
-    if hasattr(istr, 'fileno'):
-        try:
-            istr.fileno()
-            return True
-        except IOError:
-            return False
-    return False
+    if not hasattr(istr, 'fileno'):
+        return False
+    try:
+        istr.fileno()
+    except IOError:
+        return False
+    else:
+        return True
+
+
+def get_twisted_child_with_default(res, path, request):
+    # this hack is necessary because twisted takes the slash character in
+    # http requests too seriously. i.e. it insists that a leaf node can only
+    # handle the last path fragment.
+    if res.prepath is None:
+        request.realprepath = b'/' + b'/'.join(request.prepath)
+    else:
+        if not res.prepath.startswith('/'):
+            request.realprepath = b'/' + res.prepath
+        else:
+            request.realprepath = res.prepath
+
+    if path in res.children:
+        retval = res.children[path]
+    else:
+        retval = res.getChild(path, request)
+
+    if isinstance(retval, NoResource):
+        retval = res
+    else:
+        request.realpostpath = request.path[
+                               len(path) + (0 if path.startswith(b'/') else 1):]
+
+    return retval
 
 
 class TwistedWebResource(Resource):
@@ -316,38 +470,16 @@ class TwistedWebResource(Resource):
         self.prepath = prepath
 
     def getChildWithDefault(self, path, request):
-        # this hack is necessary because twisted takes the slash character in
-        # http requests too seriously. i.e. it insists that a leaf node can only
-        # handle the last path fragment.
-        if self.prepath is None:
-            request.realprepath = '/' + '/'.join(request.prepath)
-        else:
-            if not self.prepath.startswith('/'):
-                request.realprepath = '/' + self.prepath
-            else:
-                request.realprepath = self.prepath
-
-        if path in self.children:
-            retval = self.children[path]
-        else:
-            retval = self.getChild(path, request)
-
-        if isinstance(retval, NoResource):
-            retval = self
-        else:
-            request.realpostpath = request.path[len(request.realprepath):]
-
-        return retval
+        return get_twisted_child_with_default(self, path, request)
 
     def render(self, request):
-        if request.method == 'GET' and (
-                request.uri.endswith('.wsdl') or request.uri.endswith('?wsdl')):
+        if request.method == b'GET' and (
+              request.uri.endswith(b'.wsdl') or request.uri.endswith(b'?wsdl')):
             return self.__handle_wsdl_request(request)
         return self.handle_rpc(request)
 
     def handle_rpc_error(self, p_ctx, others, error, request):
         logger.error(error)
-
         resp_code = p_ctx.transport.resp_code
         # If user code set its own response code, don't touch it.
         if resp_code is None:
@@ -363,7 +495,7 @@ class TwistedWebResource(Resource):
         p_ctx.out_object = error
         self.http_transport.get_out_string(p_ctx)
 
-        retval = ''.join(p_ctx.out_string)
+        retval = b''.join(p_ctx.out_string)
 
         p_ctx.close()
 
@@ -374,6 +506,7 @@ class TwistedWebResource(Resource):
     def handle_rpc(self, request):
         initial_ctx = TwistedHttpMethodContext(self.http_transport, request,
                                  self.http_transport.app.out_protocol.mime_type)
+
         if _has_fd(request.content):
             f = request.content
 
@@ -386,12 +519,16 @@ class TwistedWebResource(Resource):
             request.content.seek(0)
             initial_ctx.in_string = [request.content.read()]
 
-        initial_ctx.transport.file_info = \
-                                        _get_file_name(initial_ctx.in_string[0])
+        initial_ctx.transport.file_info = _get_file_info(initial_ctx)
 
         contexts = self.http_transport.generate_contexts(initial_ctx)
         p_ctx, others = contexts[0], contexts[1:]
 
+        p_ctx.active = True
+        p_ctx.out_stream = request
+        # TODO: Rate limiting
+        p_ctx.active = True
+
         if p_ctx.in_error:
             return self.handle_rpc_error(p_ctx, others, p_ctx.in_error, request)
 
@@ -410,14 +547,24 @@ class TwistedWebResource(Resource):
         ret = p_ctx.out_object[0]
         retval = NOT_DONE_YET
         if isinstance(ret, Deferred):
-            ret.addCallback(_cb_deferred, request, p_ctx, others, self)
-            ret.addErrback(_eb_deferred, request, p_ctx, others, self)
+            ret.addCallback(_cb_deferred, request, p_ctx, others, resource=self)
+            ret.addErrback(_eb_deferred, request, p_ctx, others, resource=self)
+            ret.addErrback(log_and_let_go, logger)
 
         elif isinstance(ret, PushBase):
-            _init_push(ret, request, p_ctx, others, self)
+            self.http_transport.init_root_push(ret, p_ctx, others)
 
         else:
-            retval = _cb_deferred(p_ctx.out_object, request, p_ctx, others, self, cb=False)
+            try:
+                retval = _cb_deferred(p_ctx.out_object, request, p_ctx, others,
+                                                                 self, cb=False)
+            except Exception as e:
+                logger_server.exception(e)
+                try:
+                    _eb_deferred(Failure(), request, p_ctx, others,
+                                                                  resource=self)
+                except Exception as e:
+                    logger_server.exception(e)
 
         return retval
 
@@ -467,48 +614,10 @@ def _eb_request_finished(retval, request, p_ctx):
     request.finish()
 
 
-def _init_push(ret, request, p_ctx, others, resource):
-    assert isinstance(ret, PushBase)
-
-    # fire events
-    p_ctx.app.event_manager.fire_event('method_return_push', p_ctx)
-    if p_ctx.service_class is not None:
-        p_ctx.service_class.event_manager.fire_event('method_return_push', p_ctx)
-
-    gen = resource.http_transport.get_out_string_push(p_ctx)
-
-    assert isgenerator(gen), "It looks like this protocol is not " \
-                             "async-compliant yet."
-
-    def _cb_push_finish():
-        p_ctx.out_stream.finish()
-        process_contexts(resource.http_transport, others, p_ctx)
-
-    retval = ret.init(p_ctx, request, gen, _cb_push_finish, None)
-
-    if isinstance(retval, Deferred):
-        def _eb_push_close(f):
-            ret.close()
-
-        def _cb_push_close(r):
-            def _eb_inner(f):
-                return f
-
-            if r is None:
-                ret.close()
-            else:
-                r.addCallback(_cb_push_close).addErrback(_eb_inner)
-
-        retval.addCallback(_cb_push_close).addErrback(_eb_push_close)
-
-    else:
-        ret.close()
-
-    return retval
-
-
 def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
+    ### set response headers
     resp_code = p_ctx.transport.resp_code
+
     # If user code set its own response code, don't touch it.
     if resp_code is None:
         resp_code = HTTP_200
@@ -516,10 +625,11 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
 
     _set_response_headers(request, p_ctx.transport.resp_headers)
 
+    ### normalize response data
     om = p_ctx.descriptor.out_message
     single_class = None
     if cb:
-        if p_ctx.descriptor.body_style in (BODY_STYLE_BARE, BODY_STYLE_EMPTY):
+        if p_ctx.descriptor.is_out_bare():
             p_ctx.out_object = [ret]
 
         elif (not issubclass(om, ComplexModelBase)) or len(om._type_info) <= 1:
@@ -531,11 +641,11 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
     else:
         p_ctx.out_object = ret
 
+    ### start response
     retval = NOT_DONE_YET
 
-    p_ctx.out_stream = request
     if isinstance(ret, PushBase):
-        retval = _init_push(ret, request, p_ctx, others, resource)
+        resource.http_transport.init_root_push(ret, p_ctx, others)
 
     elif ((isclass(om) and issubclass(om, File)) or
           (isclass(single_class) and issubclass(single_class, File))) and \
@@ -553,17 +663,45 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
             def _close_only_context(ret):
                 p_ctx.close()
 
-            request.notifyFinish().addCallback(_close_only_context)
-            request.notifyFinish().addErrback(_eb_request_finished, request, p_ctx)
+            request.notifyFinish() \
+                .addCallback(_close_only_context) \
+                .addErrback(_eb_request_finished, request, p_ctx) \
+                .addErrback(log_and_let_go, logger)
 
     else:
-        resource.http_transport.get_out_string(p_ctx)
+        ret = resource.http_transport.get_out_string(p_ctx)
 
-        producer = Producer(p_ctx.out_string, request)
-        producer.deferred.addCallback(_cb_request_finished, request, p_ctx)
-        producer.deferred.addErrback(_eb_request_finished, request, p_ctx)
+        if not isinstance(ret, Deferred):
+            producer = Producer(p_ctx.out_string, request)
+            producer.deferred \
+                .addCallback(_cb_request_finished, request, p_ctx) \
+                .addErrback(_eb_request_finished, request, p_ctx) \
+                .addErrback(log_and_let_go, logger)
 
-        request.registerProducer(producer, False)
+            try:
+                request.registerProducer(producer, False)
+            except Exception as e:
+                logger_server.exception(e)
+                try:
+                    _eb_deferred(Failure(), request, p_ctx, others, resource)
+                except Exception as e:
+                    logger_server.exception(e)
+                    raise
+
+        else:
+            def _cb(ret):
+                if isinstance(ret, Deferred):
+                    return ret \
+                        .addCallback(_cb) \
+                        .addErrback(_eb_request_finished, request, p_ctx) \
+                        .addErrback(log_and_let_go, logger)
+                else:
+                    return _cb_request_finished(ret, request, p_ctx)
+
+            ret \
+                .addCallback(_cb) \
+                .addErrback(_eb_request_finished, request, p_ctx) \
+                .addErrback(log_and_let_go, logger)
 
     process_contexts(resource.http_transport, others, p_ctx)
 
@@ -571,10 +709,8 @@ def _cb_deferred(ret, request, p_ctx, others, resource, cb=True):
 
 
 def _eb_deferred(ret, request, p_ctx, others, resource):
-    app = p_ctx.app
-
     # DRY this with what's in Application.process_request
-    if issubclass(ret.type, Redirect):
+    if ret.check(Redirect):
         try:
             ret.value.do_redirect()
 
@@ -584,44 +720,31 @@ def _eb_deferred(ret, request, p_ctx, others, resource):
 
             _cb_deferred(None, request, p_ctx, others, resource, cb=False)
 
-            # fire events
-            app.event_manager.fire_event('method_redirect', p_ctx)
-            if p_ctx.service_class is not None:
-                p_ctx.service_class.event_manager.fire_event(
-                    'method_redirect', p_ctx)
+            p_ctx.fire_event('method_redirect')
 
         except Exception as e:
             logger_server.exception(e)
             p_ctx.out_error = Fault('Server', get_fault_string_from_exception(e))
 
-            # fire events
-            app.event_manager.fire_event('method_redirect_exception', p_ctx)
-            if p_ctx.service_class is not None:
-                p_ctx.service_class.event_manager.fire_event(
-                    'method_redirect_exception', p_ctx)
+            p_ctx.fire_event('method_redirect_exception')
 
-    elif issubclass(ret.type, Fault):
+    elif ret.check(Fault):
         p_ctx.out_error = ret.value
 
         ret = resource.handle_rpc_error(p_ctx, others, p_ctx.out_error, request)
 
-        # fire events
-        app.event_manager.fire_event('method_exception_object', p_ctx)
-        if p_ctx.service_class is not None:
-            p_ctx.service_class.event_manager.fire_event(
-                                               'method_exception_object', p_ctx)
+        p_ctx.fire_event('method_exception_object')
 
         request.write(ret)
 
     else:
-        p_ctx.out_error = ret.value
-        ret.printTraceback()
         p_ctx.out_error = InternalError(ret.value)
+        logger.error(ret.getTraceback())
 
-        # fire events
-        app.event_manager.fire_event('method_exception_object', p_ctx)
-        if p_ctx.service_class is not None:
-            p_ctx.service_class.event_manager.fire_event(
-                                               'method_exception_object', p_ctx)
+        ret = resource.handle_rpc_error(p_ctx, others, p_ctx.out_error, request)
+
+        p_ctx.fire_event('method_exception_object')
+
+        request.write(ret)
 
     request.finish()
