@@ -30,13 +30,14 @@ from lxml import etree
 from base64 import b64encode
 from decimal import Decimal as D
 
-from spyne import Application, rpc, mrpc, ServiceBase, ByteArray, Array, \
+from spyne import Application, rpc, mrpc, Service, ByteArray, Array, \
     ComplexModel, SelfReference, XmlData, XmlAttribute, Unicode, DateTime, \
     Float, Integer, String
-from spyne.const import xml_ns
+from spyne.const import xml
 from spyne.error import ResourceNotFoundError
 from spyne.interface import Interface
 from spyne.interface.wsdl import Wsdl11
+from spyne.model.addtl import TimeSegment, DateSegment, DateTimeSegment
 from spyne.protocol import ProtocolBase
 from spyne.protocol.soap import Soap11
 from spyne.server.null import NullServer
@@ -104,6 +105,29 @@ Level1.resolve_namespace(Level1, __name__)
 
 
 class TestComplexModel(unittest.TestCase):
+    def test_validate_on_assignment_fail(self):
+        class C(ComplexModel):
+            i = Integer(voa=True)
+
+        try:
+            C().i = 'a'
+        except ValueError:
+            pass
+        else:
+            raise Exception('must fail with ValueError')
+
+    def test_validate_on_assignment_success(self):
+        class C(ComplexModel):
+            i = Integer(voa=True)
+
+        c = C()
+
+        c.i = None
+        assert c.i is None
+
+        c.i = 5
+        assert c.i == 5
+
     def test_simple_class(self):
         a = Address()
         a.street = '123 happy way'
@@ -115,22 +139,22 @@ class TestComplexModel(unittest.TestCase):
         element = etree.Element('test')
         XmlDocument().to_parent(None, Address, a, element, ns_test)
         element = element[0]
-        self.assertEquals(5, len(element.getchildren()))
+        self.assertEqual(5, len(element.getchildren()))
 
         a.since = datetime.datetime(year=2011, month=12, day=31, tzinfo=pytz.utc)
         element = etree.Element('test')
         XmlDocument().to_parent(None, Address, a, element, ns_test)
         element = element[0]
-        self.assertEquals(6, len(element.getchildren()))
+        self.assertEqual(6, len(element.getchildren()))
 
         r = XmlDocument().from_element(None, Address, element)
 
-        self.assertEquals(a.street, r.street)
-        self.assertEquals(a.city, r.city)
-        self.assertEquals(a.zip, r.zip)
-        self.assertEquals(a.lattitude, r.lattitude)
-        self.assertEquals(a.longitude, r.longitude)
-        self.assertEquals(a.since, r.since)
+        self.assertEqual(a.street, r.street)
+        self.assertEqual(a.city, r.city)
+        self.assertEqual(a.zip, r.zip)
+        self.assertEqual(a.lattitude, r.lattitude)
+        self.assertEqual(a.longitude, r.longitude)
+        self.assertEqual(a.since, r.since)
 
     def test_nested_class(self): # FIXME: this test is incomplete
         p = Person()
@@ -138,10 +162,10 @@ class TestComplexModel(unittest.TestCase):
         XmlDocument().to_parent(None, Person, p, element, ns_test)
         element = element[0]
 
-        self.assertEquals(None, p.name)
-        self.assertEquals(None, p.birthdate)
-        self.assertEquals(None, p.age)
-        self.assertEquals(None, p.addresses)
+        self.assertEqual(None, p.name)
+        self.assertEqual(None, p.birthdate)
+        self.assertEqual(None, p.age)
+        self.assertEqual(None, p.addresses)
 
     def test_class_array(self):
         peeps = []
@@ -162,12 +186,12 @@ class TestComplexModel(unittest.TestCase):
         XmlDocument().to_parent(None, type, peeps, element, ns_test)
         element = element[0]
 
-        self.assertEquals(4, len(element.getchildren()))
+        self.assertEqual(4, len(element.getchildren()))
 
         peeps2 = XmlDocument().from_element(None, type, element)
         for i in range(0, 4):
-            self.assertEquals(peeps2[i].name, names[i])
-            self.assertEquals(peeps2[i].birthdate, dob)
+            self.assertEqual(peeps2[i].name, names[i])
+            self.assertEqual(peeps2[i].birthdate, dob)
 
     def test_class_nested_array(self):
         peeps = []
@@ -193,13 +217,13 @@ class TestComplexModel(unittest.TestCase):
         XmlDocument().to_parent(None, arr, peeps, element, ns_test)
         element = element[0]
 
-        self.assertEquals(4, len(element.getchildren()))
+        self.assertEqual(4, len(element.getchildren()))
 
         peeps2 = XmlDocument().from_element(None, arr, element)
         for peep in peeps2:
-            self.assertEquals(27, peep.age)
-            self.assertEquals(25, len(peep.addresses))
-            self.assertEquals('funkytown', peep.addresses[18].city)
+            self.assertEqual(27, peep.age)
+            self.assertEqual(25, len(peep.addresses))
+            self.assertEqual('funkytown', peep.addresses[18].city)
 
     def test_complex_class(self):
         l = Level1()
@@ -224,10 +248,10 @@ class TestComplexModel(unittest.TestCase):
         element = element[0]
         l1 = XmlDocument().from_element(None, Level1, element)
 
-        self.assertEquals(l1.level2.arg1, l.level2.arg1)
-        self.assertEquals(l1.level2.arg2, l.level2.arg2)
-        self.assertEquals(len(l1.level4), len(l.level4))
-        self.assertEquals(100, len(l.level3))
+        self.assertEqual(l1.level2.arg1, l.level2.arg1)
+        self.assertEqual(l1.level2.arg2, l.level2.arg2)
+        self.assertEqual(len(l1.level4), len(l.level4))
+        self.assertEqual(100, len(l.level3))
 
 
 class X(ComplexModel):
@@ -293,6 +317,73 @@ class EncExtractXs(ComplexModel):
     enc_idn = Integer(nillable=False, min_occurs=1, max_occurs=1, max_len=18)
     hist_idn = Integer(nillable=False, min_occurs=1, max_occurs=1, max_len=18)
 
+class TestComplex(unittest.TestCase):
+    def test_array_type_name(self):
+        assert Array(String, type_name='punk').__type_name__ == 'punk'
+
+    def test_ctor_kwargs(self):
+        class Category(ComplexModel):
+            id = Integer(min_occurs=1, max_occurs=1, nillable=False)
+            children = Array(Unicode)
+
+        v = Category(id=5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+    def test_ctor_args(self):
+        class Category(ComplexModel):
+            id = XmlData(Integer(min_occurs=1, max_occurs=1, nillable=False))
+            children = Array(Unicode)
+
+        v = Category(id=5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+        v = Category(5, children=['a','b'])
+
+        assert v.id == 5
+        assert v.children == ['a', 'b']
+
+    def test_ctor_args_2(self):
+        class Category(ComplexModel):
+            children = Array(Unicode)
+
+        class BetterCategory(Category):
+            sub_category = Unicode
+
+        v = BetterCategory(children=['a','b'], sub_category='aaa')
+
+        assert v.children == ['a', 'b']
+        assert v.sub_category == 'aaa'
+
+    def test_flat_type_info(self):
+        class A(ComplexModel):
+            i = Integer
+
+        class B(A):
+            s = String
+
+        assert 's' in B.get_flat_type_info(B)
+        assert 'i' in B.get_flat_type_info(B)
+
+    def test_flat_type_info_attr(self):
+        class A(ComplexModel):
+            i = Integer
+            ia = XmlAttribute(Integer)
+
+        class B(A):
+            s = String
+            sa = XmlAttribute(String)
+
+        assert 's' in B.get_flat_type_info(B)
+        assert 'i' in B.get_flat_type_info(B)
+        assert 'sa' in B.get_flat_type_info(B)
+        assert 'ia' in B.get_flat_type_info(B)
+        assert 'sa' in B.get_flat_type_info(B).attrs
+        assert 'ia' in B.get_flat_type_info(B).attrs
+
 
 class TestXmlAttribute(unittest.TestCase):
     def assertIsNotNone(self, obj, msg=None):
@@ -318,7 +409,7 @@ class TestXmlAttribute(unittest.TestCase):
         wsdl.build_interface_document('http://a-aaaa.com')
         pref = CM.get_namespace_prefix(interface)
         type_def = wsdl.get_schema_info(pref).types[CM.get_type_name()]
-        attribute_def = type_def.find('{%s}attribute' % xml_ns.xsd)
+        attribute_def = type_def.find(xml.XSD('attribute'))
         print(etree.tostring(type_def, pretty_print=True))
 
         self.assertIsNotNone(attribute_def)
@@ -330,10 +421,10 @@ class TestXmlAttribute(unittest.TestCase):
             __namespace__ = 'myns'
             Data = ByteArray
 
-        test_string = 'yo test data'
+        test_string = b'yo test data'
         b64string = b64encode(test_string)
 
-        gg = PacketNonAttribute(Data=test_string)
+        gg = PacketNonAttribute(Data=[test_string])
 
         element = etree.Element('test')
         Soap11().to_parent(None, PacketNonAttribute, gg, element, gg.get_namespace())
@@ -341,7 +432,7 @@ class TestXmlAttribute(unittest.TestCase):
         element = element[0]
         #print etree.tostring(element, pretty_print=True)
         data = element.find('{%s}Data' % gg.get_namespace()).text
-        self.assertEquals(data, b64string)
+        self.assertEqual(data, b64string.decode('ascii'))
         s1 = Soap11().from_element(None, PacketNonAttribute, element)
         assert s1.Data[0] == test_string
 
@@ -350,16 +441,17 @@ class TestXmlAttribute(unittest.TestCase):
             __namespace__ = 'myns'
             Data = XmlAttribute(ByteArray, use='required')
 
-        test_string = 'yo test data'
+        test_string = b'yo test data'
         b64string = b64encode(test_string)
-        gg = PacketAttribute(Data=test_string)
+        gg = PacketAttribute(Data=[test_string])
 
         element = etree.Element('test')
         Soap11().to_parent(None, PacketAttribute, gg, element, gg.get_namespace())
 
         element = element[0]
         print(etree.tostring(element, pretty_print=True))
-        self.assertEquals(element.attrib['Data'], b64string)
+        print(element.attrib)
+        self.assertEqual(element.attrib['Data'], b64string.decode('ascii'))
 
         s1 = Soap11().from_element(None, PacketAttribute, element)
         assert s1.Data[0] == test_string
@@ -367,7 +459,7 @@ class TestXmlAttribute(unittest.TestCase):
     def test_customized_type(self):
         class SomeClass(ComplexModel):
             a = XmlAttribute(Integer(ge=4))
-        class SomeService(ServiceBase):
+        class SomeService(Service):
             @rpc(SomeClass)
             def some_call(ctx, some_class):
                 pass
@@ -513,7 +605,7 @@ class TestSelfRefence(unittest.TestCase):
 
         assert c is TestSelfReference
 
-        class SoapService(ServiceBase):
+        class SoapService(Service):
             @rpc(_returns=TestSelfReference)
             def view_categories(ctx):
                 pass
@@ -535,7 +627,7 @@ class TestSelfRefence(unittest.TestCase):
         assert d['children'][0]['id'] == 0
         assert d['children'][1]['id'] == 1
 
-        class SoapService(ServiceBase):
+        class SoapService(Service):
             @rpc(_returns=Category)
             def view_categories(ctx):
                 pass
@@ -555,46 +647,6 @@ class TestSelfRefence(unittest.TestCase):
         sr, = Category._type_info['children']._type_info.values()
         assert issubclass(sr, Category)
 
-    def test_array_type_name(self):
-        assert Array(String, type_name='punk').__type_name__ == 'punk'
-
-    def test_ctor_kwargs(self):
-        class Category(ComplexModel):
-            id = Integer(min_occurs=1, max_occurs=1, nillable=False)
-            children = Array(Unicode)
-
-        v = Category(id=5, children=['a','b'])
-
-        assert v.id == 5
-        assert v.children == ['a', 'b']
-
-    def test_ctor_args(self):
-        class Category(ComplexModel):
-            id = XmlData(Integer(min_occurs=1, max_occurs=1, nillable=False))
-            children = Array(Unicode)
-
-        v = Category(id=5, children=['a','b'])
-
-        assert v.id == 5
-        assert v.children == ['a', 'b']
-
-        v = Category(5, children=['a','b'])
-
-        assert v.id == 5
-        assert v.children == ['a', 'b']
-
-    def test_ctor_args_2(self):
-        class Category(ComplexModel):
-            children = Array(Unicode)
-
-        class BetterCategory(Category):
-            sub_category = Unicode
-
-        v = BetterCategory(children=['a','b'], sub_category='aaa')
-
-        assert v.children == ['a', 'b']
-        assert v.sub_category == 'aaa'
-
 
 class TestMemberRpc(unittest.TestCase):
     def test_simple(self):
@@ -659,14 +711,14 @@ class TestMemberRpc(unittest.TestCase):
         print(methods)
         assert 'member_method' in methods
 
-        class SomeService(ServiceBase):
+        class SomeService(Service):
             @rpc(_returns=SomeComplexModel)
             def service_method(ctx):
                 return SomeComplexModel()
 
         app = Application([SomeService], 'some_ns')
 
-        mmm = __name__ + '.SomeComplexModel.SomeComplexModel.member_method'
+        mmm = __name__ + '.SomeComplexModel.member_method'
         assert mmm in app.interface.method_id_map
 
     def test_interface_mult(self):
@@ -679,7 +731,7 @@ class TestMemberRpc(unittest.TestCase):
         print(methods)
         assert 'member_method' in methods
 
-        class SomeService(ServiceBase):
+        class SomeService(Service):
             @rpc(_returns=SomeComplexModel)
             def service_method(ctx):
                 return SomeComplexModel()
@@ -690,7 +742,7 @@ class TestMemberRpc(unittest.TestCase):
 
         app = Application([SomeService], 'some_ns')
 
-        mmm = __name__ + '.SomeComplexModel.SomeComplexModel.member_method'
+        mmm = __name__ + '.SomeComplexModel.member_method'
         assert mmm in app.interface.method_id_map
 
     def test_remote_call_error(self):
@@ -702,7 +754,7 @@ class TestMemberRpc(unittest.TestCase):
             def put(self, ctx):
                 return v
 
-        class SomeService(ServiceBase):
+        class SomeService(Service):
             @rpc(_returns=SomeComplexModel)
             def get(ctx):
                 return SomeComplexModel()
@@ -748,7 +800,7 @@ class TestMemberRpc(unittest.TestCase):
             def echo(self, ctx):
                 return self
 
-        class SomeService(ServiceBase):
+        class SomeService(Service):
             @rpc(_returns=SomeComplexModel)
             def get(ctx):
                 return SomeComplexModel()
@@ -756,7 +808,7 @@ class TestMemberRpc(unittest.TestCase):
         null = NullServer(Application([SomeService], tns='some_tns'))
 
         v = SomeComplexModel(i=5)
-        assert null.service['SomeComplexModel.SomeComplexModel.echo'](v) is v
+        assert null.service['SomeComplexModel.echo'](v) is v
 
     def test_order(self):
         class CM(ComplexModel):
@@ -971,9 +1023,34 @@ class TestCustomize(unittest.TestCase):
         B2 = B.customize(child_attrs=dict(s=dict(max_len=10)))
         assert B2.get_flat_type_info(B2)['s'].Attributes.max_len == 10
 
-        B3 = B2.customize(child_attrs=dict(d=dict(format="%y")))
+        B3 = B2.customize(child_attrs=dict(d=dict(dt_format="%y")))
         assert B3.get_flat_type_info(B3)['s'].Attributes.max_len == 10
 
+    def test_cust_all(self):
+        class A(ComplexModel):
+            s = Unicode
+            i = Unicode
+
+        class B(A):
+            d = DateTime
+
+        B2 = B.customize(child_attrs_all=dict(max_len=10))
+        assert B2.get_flat_type_info(B2)['s'].Attributes.max_len == 10
+        assert B2.get_flat_type_info(B2)['i'].Attributes.max_len == 10
+
+    def test_cust_noexc(self):
+        class A(ComplexModel):
+            s = Unicode
+            i = Integer
+
+        class B(A):
+            d = DateTime
+
+        B2 = B.customize(child_attrs_noexc=dict(s=dict(max_len=10)))
+        assert B2.get_flat_type_info(B2)['s'].Attributes.max_len == 10
+        assert B2.get_flat_type_info(B2)['s'].Attributes.exc == False
+        assert B2.get_flat_type_info(B2)['i'].Attributes.exc == True
+
 
     def test_complex_type_name_clashes(self):
         class TestComplexModel(ComplexModel):
@@ -986,7 +1063,7 @@ class TestCustomize(unittest.TestCase):
 
         TestComplexModel2 = TestComplexModel
 
-        class TestService(ServiceBase):
+        class TestService(Service):
             @rpc(TestComplexModel1)
             def test1(ctx, obj):
                 pass
@@ -1008,5 +1085,32 @@ class TestCustomize(unittest.TestCase):
                     "have conflicting names.")
 
 
+class TestAdditional(unittest.TestCase):
+    def test_time_segment(self):
+        data = TimeSegment.from_string("[11:12:13.123456,14:15:16.789012]")
+
+        assert data.start_inclusive
+        assert data.start == datetime.time(11, 12, 13, 123456)
+        assert data.end == datetime.time(14, 15, 16, 789012)
+        assert data.end_inclusive
+
+    def test_date_segment(self):
+        data = DateSegment.from_string("[2016-03-03,2016-05-07[")
+
+        assert data.start_inclusive == True
+        assert data.start == datetime.date(2016, 3, 3)
+        assert data.end == datetime.date(2016, 5, 7)
+        assert data.end_inclusive == False
+
+    def test_datetime_segment(self):
+        data = DateTimeSegment.from_string("]2016-03-03T10:20:30.405060,"
+                                            "2016-05-07T00:01:02.030405]")
+
+        assert data.start_inclusive == False
+        assert data.start == datetime.datetime(2016, 3, 3, 10, 20, 30, 405060)
+        assert data.end   == datetime.datetime(2016, 5, 7,  0,  1,  2,  30405)
+        assert data.end_inclusive == True
+
+
 if __name__ == '__main__':
     unittest.main()
