@@ -20,9 +20,12 @@
 import logging
 logging.basicConfig(level=logging.DEBUG)
 
+import inspect
 import unittest
 import sqlalchemy
 
+from pprint import pprint
+
 from sqlalchemy import create_engine
 from sqlalchemy import MetaData
 from sqlalchemy import Column
@@ -32,24 +35,106 @@ from sqlalchemy.exc import IntegrityError
 from sqlalchemy.orm import mapper
 from sqlalchemy.orm import sessionmaker
 
-from spyne import M
+from spyne import M, Any, Double
 
-from spyne.model import XmlAttribute, File
-from spyne.model import XmlData
-from spyne.model import ComplexModel
-from spyne.model import Array
-from spyne.model import Integer32
-from spyne.model import Unicode
-from spyne.model import Integer
-from spyne.model import Enum
-from spyne.model import TTableModel
+from spyne.model import XmlAttribute, File, XmlData, ComplexModel, Array, \
+    Integer32, Unicode, Integer, Enum, TTableModel, DateTime, Boolean
 
 from spyne.model.binary import HybridFileStore
 from spyne.model.complex import xml
 from spyne.model.complex import table
 
+from spyne.store.relational import get_pk_columns
+from spyne.store.relational.document import PGJsonB, PGJson, PGFileJson, \
+    PGObjectJson
+
 TableModel = TTableModel()
 
+
+class TestSqlAlchemyTypeMappings(unittest.TestCase):
+    def test_init(self):
+        fn = inspect.stack()[0][3]
+        from sqlalchemy.inspection import inspect as sqla_inspect
+
+        class SomeClass1(TableModel):
+            __tablename__ = "%s_%d" % (fn, 1)
+            i = Integer32(pk=True)
+            e = Unicode(32)
+
+        from spyne.util.dictdoc import get_dict_as_object
+        inst = get_dict_as_object(dict(i=4), SomeClass1)
+        assert not sqla_inspect(inst).attrs.e.history.has_changes()
+
+    def test_bool(self):
+        fn = inspect.stack()[0][3]
+
+        class SomeClass1(TableModel):
+            __tablename__ = "%s_%d" % (fn, 1)
+            i = Integer32(pk=True)
+            b = Boolean
+
+        assert isinstance(SomeClass1.Attributes.sqla_table.c.b.type,
+                                                             sqlalchemy.Boolean)
+
+        class SomeClass2(TableModel):
+            __tablename__ = "%s_%d" % (fn, 2)
+            i = Integer32(pk=True)
+            b = Boolean(store_as=int)
+
+        assert isinstance(SomeClass2.Attributes.sqla_table.c.b.type,
+                                                        sqlalchemy.SmallInteger)
+
+    def test_jsonb(self):
+        fn = inspect.stack()[0][3]
+
+        class SomeClass1(TableModel):
+            __tablename__ = "%s_%d" % (fn, 1)
+            i = Integer32(pk=True)
+            a = Any(store_as='json')
+
+        assert isinstance(SomeClass1.Attributes.sqla_table.c.a.type, PGJson)
+
+        class SomeClass2(TableModel):
+            __tablename__ = "%s_%d" % (fn, 2)
+            i = Integer32(pk=True)
+            a = Any(store_as='jsonb')
+
+        assert isinstance(SomeClass2.Attributes.sqla_table.c.a.type, PGJsonB)
+
+        class SomeClass3(TableModel):
+            __tablename__ = "%s_%d" % (fn, 3)
+            i = Integer32(pk=True)
+            a = File(store_as=HybridFileStore("path", db_format='jsonb'))
+
+        assert isinstance(SomeClass3.Attributes.sqla_table.c.a.type, PGFileJson)
+        assert SomeClass3.Attributes.sqla_table.c.a.type.dbt == 'jsonb'
+
+    def test_obj_json(self):
+        fn = inspect.stack()[0][3]
+
+        class SomeClass(ComplexModel):
+            s = Unicode
+            d = Double
+
+        class SomeClass1(TableModel):
+            __tablename__ = "%s_%d" % (fn, 1)
+            _type_info = [
+                ('i', Integer32(pk=True)),
+                ('a', Array(SomeClass, store_as='json')),
+            ]
+
+        assert isinstance(SomeClass1.Attributes.sqla_table.c.a.type,
+                                                                   PGObjectJson)
+
+        class SomeClass2(TableModel):
+            __tablename__ = "%s_%d" % (fn, 2)
+            i = Integer32(pk=True)
+            a = SomeClass.customize(store_as='json')
+
+        assert isinstance(SomeClass2.Attributes.sqla_table.c.a.type,
+                                                                   PGObjectJson)
+
+
 class TestSqlAlchemySchema(unittest.TestCase):
     def setUp(self):
         logging.getLogger('sqlalchemy').setLevel(logging.DEBUG)
@@ -58,6 +143,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         self.session = sessionmaker(bind=self.engine)()
         self.metadata = TableModel.Attributes.sqla_metadata = MetaData()
         self.metadata.bind = self.engine
+        logging.info('Testing against sqlalchemy-%s', sqlalchemy.__version__)
 
     def test_schema(self):
         class SomeClass(TableModel):
@@ -69,17 +155,76 @@ class TestSqlAlchemySchema(unittest.TestCase):
             i = Integer32(64, index=True)
 
         t = SomeClass.__table__
-        self.metadata.create_all() # not needed, just nice to see.
+        self.metadata.create_all()  # not needed, just nice to see.
 
         assert t.c.id.primary_key == True
         assert t.c.id.autoincrement == False
         indexes = list(t.indexes)
-        indexes.sort(key=lambda idx: idx.columns)
+        indexes.sort(key=lambda idx: idx.name)
         for idx in indexes:
             assert 'i' in idx.columns or 's' in idx.columns
             if 's' in idx.columns:
                 assert idx.unique
 
+    def test_colname_simple(self):
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True, autoincrement=False)
+            s = Unicode(64, sqla_column_args=dict(name='ss'))
+
+        t = SomeClass.__table__
+        self.metadata.create_all()  # not needed, just nice to see.
+
+        assert 'ss' in t.c
+
+    def test_colname_complex_table(self):
+        class SomeOtherClass(TableModel):
+            __tablename__ = 'some_other_class'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+            __table_args__ = (
+                {"sqlite_autoincrement": True},
+            )
+
+            id = Integer32(primary_key=True)
+            o = SomeOtherClass.customize(store_as='table',
+                                               sqla_column_args=dict(name='oo'))
+
+        t = SomeClass.__table__
+        self.metadata.create_all()  # not needed, just nice to see.
+
+        assert 'oo_id' in t.c
+
+    def test_colname_complex_json(self):
+        class SomeOtherClass(TableModel):
+            __tablename__ = 'some_other_class'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+            __table_args__ = (
+                {"sqlite_autoincrement": True},
+            )
+
+            id = Integer32(primary_key=True)
+            o = SomeOtherClass.customize(store_as='json',
+                                               sqla_column_args=dict(name='oo'))
+
+        t = SomeClass.__table__
+        self.metadata.create_all()  # not needed, just nice to see.
+
+        assert 'oo' in t.c
+
     def test_nested_sql(self):
         class SomeOtherClass(TableModel):
             __tablename__ = 'some_other_class'
@@ -196,7 +341,8 @@ class TestSqlAlchemySchema(unittest.TestCase):
             __table_args__ = {"sqlite_autoincrement": True}
 
             id = Integer32(primary_key=True)
-            others = Array(SomeOtherClass, store_as=table(multi=True, backref='some_classes'))
+            others = Array(SomeOtherClass,
+                             store_as=table(multi=True, backref='some_classes'))
 
         self.metadata.create_all()
 
@@ -266,8 +412,8 @@ class TestSqlAlchemySchema(unittest.TestCase):
         self.session.commit()
         self.session.close()
 
-        sc_xml = self.session.connection().execute("select others from some_class") \
-                                                               .fetchall()[0][0]
+        sc_xml = self.session.connection() \
+                     .execute("select others from some_class") .fetchall()[0][0]
 
         from lxml import etree
         assert etree.fromstring(sc_xml).tag == 'SomeOtherClassArray'
@@ -278,6 +424,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         class SomeOtherClass(TableModel):
             __tablename__ = 'some_other_class'
             __table_args__ = {"sqlite_autoincrement": True}
+
             id = Integer32(primary_key=True)
             s = Unicode(64)
 
@@ -286,7 +433,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         self.metadata.create_all()
 
-        sc = SomeClass(id=5, s='s', numbers=[1,2,3,4])
+        sc = SomeClass(id=5, s='s', numbers=[1, 2, 3, 4])
 
         self.session.add(sc)
         self.session.commit()
@@ -307,6 +454,89 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         self.session.close()
 
+    def test_inheritance_with_complex_fields(self):
+        class Foo(TableModel):
+            __tablename__ = 'foo'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+
+        class Bar(TableModel):
+            __tablename__ = 'bar'
+            __table_args__ = {"sqlite_autoincrement": True}
+            __mapper_args__ = {
+                'polymorphic_on': 'type',
+                'polymorphic_identity': 'bar',
+                'with_polymorphic': '*',
+            }
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+            type = Unicode(6)
+            foos = Array(Foo).store_as('table')
+
+        class SubBar(Bar):
+            __mapper_args__ = {
+                'polymorphic_identity': 'subbar',
+            }
+            i = Integer32
+
+        sqlalchemy.orm.configure_mappers()
+
+        mapper_subbar = SubBar.Attributes.sqla_mapper
+        mapper_bar = Bar.Attributes.sqla_mapper
+        assert not mapper_subbar.concrete
+
+        for inheriting in mapper_subbar.iterate_to_root():
+            if inheriting is not mapper_subbar \
+                    and not (mapper_bar.relationships['foos'] is
+                                           mapper_subbar.relationships['foos']):
+                raise Exception("Thou shalt stop children relationships "
+                                           "from overriding the ones in parent")
+
+    def test_mixins_with_complex_fields(self):
+        class Foo(TableModel):
+            __tablename__ = 'foo'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+
+        class Bar(TableModel):
+            __tablename__ = 'bar'
+            __table_args__ = {"sqlite_autoincrement": True}
+            __mixin__ = True
+            __mapper_args__ = {
+                'polymorphic_on': 'type',
+                'polymorphic_identity': 'bar',
+                'with_polymorphic': '*',
+            }
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+            type = Unicode(6)
+            foos = Array(Foo).store_as('table')
+
+        class SubBar(Bar):
+            __mapper_args__ = {
+                'polymorphic_identity': 'subbar',
+            }
+            i = Integer32
+
+        sqlalchemy.orm.configure_mappers()
+
+        mapper_subbar = SubBar.Attributes.sqla_mapper
+        mapper_bar = Bar.Attributes.sqla_mapper
+        assert not mapper_subbar.concrete
+
+        for inheriting in mapper_subbar.iterate_to_root():
+            if inheriting is not mapper_subbar \
+                    and not (mapper_bar.relationships['foos'] is
+                                           mapper_subbar.relationships['foos']):
+                raise Exception("Thou shalt stop children relationships "
+                                           "from overriding the ones in parent")
+
     def test_sqlalchemy_inheritance(self):
         # no spyne code is involved here.
         # this is just to test test the sqlalchemy behavior that we rely on.
@@ -314,6 +544,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         class Employee(object):
             def __init__(self, name):
                 self.name = name
+
             def __repr__(self):
                 return self.__class__.__name__ + " " + self.name
 
@@ -321,20 +552,22 @@ class TestSqlAlchemySchema(unittest.TestCase):
             def __init__(self, name, manager_data):
                 self.name = name
                 self.manager_data = manager_data
+
             def __repr__(self):
                 return (
                     self.__class__.__name__ + " " +
-                    self.name + " " +  self.manager_data
+                    self.name + " " + self.manager_data
                 )
 
         class Engineer(Employee):
             def __init__(self, name, engineer_info):
                 self.name = name
                 self.engineer_info = engineer_info
+
             def __repr__(self):
                 return (
                     self.__class__.__name__ + " " +
-                    self.name + " " +  self.engineer_info
+                    self.name + " " + self.engineer_info
                 )
 
         employees_table = Table('employees', self.metadata,
@@ -342,15 +575,18 @@ class TestSqlAlchemySchema(unittest.TestCase):
             Column('name', sqlalchemy.String(50)),
             Column('manager_data', sqlalchemy.String(50)),
             Column('engineer_info', sqlalchemy.String(50)),
-            Column('type', sqlalchemy.String(20), nullable=False)
+            Column('type', sqlalchemy.String(20), nullable=False),
         )
 
         employee_mapper = mapper(Employee, employees_table,
-            polymorphic_on=employees_table.c.type, polymorphic_identity='employee')
+                                        polymorphic_on=employees_table.c.type,
+                                                polymorphic_identity='employee')
+
         manager_mapper = mapper(Manager, inherits=employee_mapper,
-                                            polymorphic_identity='manager')
+                                                polymorphic_identity='manager')
+
         engineer_mapper = mapper(Engineer, inherits=employee_mapper,
-                                            polymorphic_identity='engineer')
+                                                polymorphic_identity='engineer')
 
         self.metadata.create_all()
 
@@ -359,16 +595,15 @@ class TestSqlAlchemySchema(unittest.TestCase):
         self.session.commit()
         self.session.close()
 
-        assert self.session.query(Employee).with_polymorphic('*').filter_by(employee_id=1).one().type == 'manager'
+        assert self.session.query(Employee).with_polymorphic('*') \
+                   .filter_by(employee_id=1) \
+                   .one().type == 'manager'
 
     def test_inheritance_polymorphic_with_non_nullables_in_subclasses(self):
         class SomeOtherClass(TableModel):
             __tablename__ = 'some_other_class'
-            __table_args__ = {"sqlite_autoincrement": True} # this is sqlite-specific
-            __mapper_args__ = (
-                (),
-                {'polymorphic_on': 't', 'polymorphic_identity': 1},
-            )
+            __table_args__ = {"sqlite_autoincrement": True}
+            __mapper_args__ = {'polymorphic_on': 't', 'polymorphic_identity': 1}
 
             id = Integer32(primary_key=True)
             t = Integer32(nillable=False)
@@ -406,19 +641,21 @@ class TestSqlAlchemySchema(unittest.TestCase):
         else:
             raise Exception("Must fail with IntegrityError.")
 
-        sc2 = SomeClass(s='s') # this won't fail. should it?
+        sc2 = SomeClass(s='s')  # this won't fail. should it?
         self.session.add(sc2)
         self.session.commit()
 
         self.session.expunge_all()
 
-        assert self.session.query(SomeOtherClass).with_polymorphic('*').filter_by(id=soc_id).one().t == 1
+        assert self.session.query(SomeOtherClass).with_polymorphic('*') \
+                   .filter_by(id=soc_id).one().t == 1
+
         self.session.close()
 
     def test_inheritance_polymorphic(self):
         class SomeOtherClass(TableModel):
             __tablename__ = 'some_class'
-            __table_args__ = {"sqlite_autoincrement": True} # this is sqlite-specific
+            __table_args__ = {"sqlite_autoincrement": True}
             __mapper_args__ = {'polymorphic_on': 't', 'polymorphic_identity': 1}
 
             id = Integer32(primary_key=True)
@@ -431,13 +668,14 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         self.metadata.create_all()
 
-        sc = SomeClass(id=5, s='s', numbers=[1,2,3,4])
+        sc = SomeClass(id=5, s='s', numbers=[1, 2, 3, 4])
 
         self.session.add(sc)
         self.session.commit()
         self.session.close()
 
-        assert self.session.query(SomeOtherClass).with_polymorphic('*').filter_by(id=5).one().t == 2
+        assert self.session.query(SomeOtherClass).with_polymorphic('*') \
+            .filter_by(id=5).one().t == 2
         self.session.close()
 
     def test_nested_sql_array_as_json(self):
@@ -483,7 +721,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         self.session.expunge_all()
 
         ret = self.session.query(SomeClass).get(1)
-        assert ret.i == 1 # redundant
+        assert ret.i == 1  # redundant
         assert ret.s == 's'
 
     def test_default_ctor(self):
@@ -506,6 +744,25 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         assert self.session.query(SomeClass).get(1).f == 'uuu'
 
+    def test_default_value(self):
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+            __table_args__ = {"sqlite_autoincrement": True}
+
+            id = Integer32(primary_key=True)
+            f = Unicode(32, db_default=u'uuu')
+
+        self.metadata.create_all()
+        val = SomeClass()
+        assert val.f is None
+
+        self.session.add(val)
+        self.session.commit()
+
+        self.session.expunge_all()
+
+        assert self.session.query(SomeClass).get(1).f == u'uuu'
+
     def test_default_ctor_with_sql_relationship(self):
         class SomeOtherClass(TableModel):
             __tablename__ = 'some_other_class'
@@ -516,9 +773,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         class SomeClass(TableModel):
             __tablename__ = 'some_class'
-            __table_args__ = (
-                {"sqlite_autoincrement": True},
-            )
+            __table_args__ = {"sqlite_autoincrement": True}
 
             id = Integer32(primary_key=True)
             o = SomeOtherClass.customize(store_as='table')
@@ -537,9 +792,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         class SomeClass(TableModel):
             __tablename__ = 'some_class'
-            __table_args__ = (
-                {"sqlite_autoincrement": True},
-            )
+            __table_args__ = {"sqlite_autoincrement": True}
 
             id = Integer32(primary_key=True)
             o = SomeOtherClass.customize(store_as='table', index='btree')
@@ -609,9 +862,9 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         sc = self.session.query(SomeClass).get(1)
         assert ''.join([scc.s for scc in sc.children]) == 'p|q'
-        assert     sum([scc.i for scc in sc.children]) ==  619
+        assert sum([scc.i for scc in sc.children]) == 619
 
-    def test_reflection(self):
+    def test_simple_fk(self):
         class SomeChildClass(TableModel):
             __tablename__ = 'some_child_class'
 
@@ -623,12 +876,91 @@ class TestSqlAlchemySchema(unittest.TestCase):
             __tablename__ = 'some_class'
 
             id = Integer32(primary_key=True)
-            children = Array(SomeChildClass).store_as('xml')
-            mirror = SomeChildClass.store_as('json')
+            child_id = Integer32(fk='some_child_class.id')
 
-        metadata2 = MetaData()
-        metadata2.bind = self.engine
-        metadata2.reflect()
+        foreign_keys = SomeClass.__table__.c['child_id'].foreign_keys
+        assert len(foreign_keys) == 1
+        fk, = foreign_keys
+        assert fk._colspec == 'some_child_class.id'
+
+    def test_multirel_single_table(self):
+        class SomeChildClass(TableModel):
+            __tablename__ = 'some_child_class'
+
+            id = Integer32(primary_key=True)
+            s = Unicode(64)
+
+        class SomeOtherChildClass(TableModel):
+            __tablename__ = 'some_other_child_class'
+
+            id = Integer32(primary_key=True)
+            i = Integer32
+
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+
+            id = Integer32(primary_key=True)
+
+            children = Array(SomeChildClass,
+                store_as=table(
+                    multi='children', lazy='joined',
+                    left='parent_id', right='child_id',
+                    fk_left_ondelete='cascade',
+                    fk_right_ondelete='cascade',
+                ),
+            )
+
+            other_children = Array(SomeOtherChildClass,
+                store_as=table(
+                    multi='children', lazy='joined',
+                    left='parent_id', right='other_child_id',
+                    fk_left_ondelete='cascade',
+                    fk_right_ondelete='cascade',
+                ),
+            )
+
+        t = SomeClass.Attributes.sqla_metadata.tables['children']
+
+        fkp, = t.c.parent_id.foreign_keys
+        assert fkp._colspec == 'some_class.id'
+
+        fkc, = t.c.child_id.foreign_keys
+        assert fkc._colspec == 'some_child_class.id'
+
+        fkoc, = t.c.other_child_id.foreign_keys
+        assert fkoc._colspec == 'some_other_child_class.id'
+
+    def test_reflection(self):
+        class SomeClass(TableModel):
+            __tablename__ = 'some_class'
+
+            id = Integer32(primary_key=True)
+            s = Unicode(32)
+
+        TableModel.Attributes.sqla_metadata.create_all()
+
+        # create a new table model with empty metadata
+        TM2 = TTableModel()
+        TM2.Attributes.sqla_metadata.bind = self.engine
+
+        # fill it with information from the db
+        TM2.Attributes.sqla_metadata.reflect()
+
+        # convert sqla info to spyne info
+        class Reflected(TM2):
+            __table__ = TM2.Attributes.sqla_metadata.tables['some_class']
+
+        pprint(dict(Reflected._type_info).items())
+        assert issubclass(Reflected._type_info['id'], Integer)
+
+        # this looks at spyne attrs
+        assert [k for k, v in get_pk_columns(Reflected)] == ['id']
+
+        # this looks at sqla attrs
+        assert [k for k, v in Reflected.get_primary_keys()] == ['id']
+
+        assert issubclass(Reflected._type_info['s'], Unicode)
+        assert Reflected._type_info['s'].Attributes.max_len == 32
 
     def _test_sqlalchemy_remapping(self):
         class SomeTable(TableModel):
@@ -640,14 +972,15 @@ class TestSqlAlchemySchema(unittest.TestCase):
         class SomeTableSubset(TableModel):
             __table__ = SomeTable.__table__
 
-            id = Integer32(pk=True) # sqla session doesn't work without pk
+            id = Integer32(pk=True)  # sqla session doesn't work without pk
             i = Integer32
 
         class SomeTableOtherSubset(TableModel):
             __table__ = SomeTable.__table__
-            _type_info = [(k,v) for k, v in SomeTable._type_info.items() if k in ('id', 's')]
+            _type_info = [(k, v) for k, v in SomeTable._type_info.items()
+                                                            if k in ('id', 's')]
 
-        self.session.add(SomeTable(id=1,i=2,s='s'))
+        self.session.add(SomeTable(id=1, i=2, s='s'))
         self.session.commit()
 
         st = self.session.query(SomeTable).get(1)
@@ -655,7 +988,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         stos = self.session.query(SomeTableOtherSubset).get(1)
 
         sts.i = 3
-        sts.s = 'ss' # will not be flushed to db
+        sts.s = 'ss'  # will not be flushed to db
         self.session.commit()
 
         assert st.s == 's'
@@ -666,10 +999,10 @@ class TestSqlAlchemySchema(unittest.TestCase):
             __tablename__ = "c"
 
             id = Integer32(pk=True)
-            f = File(store_as=HybridFileStore('store', 'json'))
+            f = File(store_as=HybridFileStore('test_file_storage', 'json'))
 
         self.metadata.create_all()
-        c = C(f=File.Value(name="name", type="type", data=["data"]))
+        c = C(f=File.Value(name=u"name", type=u"type", data=[b"data"]))
         self.session.add(c)
         self.session.flush()
         self.session.commit()
@@ -678,7 +1011,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         print(c)
         assert c.f.name == "name"
         assert c.f.type == "type"
-        assert str(c.f.data[0][:]) == "data"
+        assert c.f.data[0][:] == b"data"
 
     def test_append_field_complex_existing_column(self):
         class C(TableModel):
@@ -693,7 +1026,21 @@ class TestSqlAlchemySchema(unittest.TestCase):
         C.append_field('d', D.store_as('table'))
         assert C.Attributes.sqla_mapper.get_property('d').argument is D
 
+    def test_append_field_complex_delayed(self):
+        class C(TableModel):
+            __tablename__ = "c"
+            u = Unicode(pk=True)
+
+        class D(C):
+            i = Integer32
+
+        C.append_field('d', DateTime)
+
+        assert D.Attributes.sqla_mapper.has_property('d')
+
     def _test_append_field_complex_explicit_existing_column(self):
+        # FIXME: Test something!
+
         class C(TableModel):
             __tablename__ = "c"
             id = Integer32(pk=True)
@@ -720,7 +1067,7 @@ class TestSqlAlchemySchema(unittest.TestCase):
         self.metadata.create_all()
 
         c1, c2 = C(id=1), C(id=2)
-        d = D(id=1, c=[c1,c2])
+        d = D(id=1, c=[c1, c2])
         self.session.add(d)
         self.session.commit()
         assert c1.d.id == 1
@@ -736,7 +1083,8 @@ class TestSqlAlchemySchema(unittest.TestCase):
 
         C.append_field('d', D.store_as('table'))
         assert C.Attributes.sqla_mapper.get_property('d').argument is D
-        assert isinstance(C.Attributes.sqla_table.c['d_id'].type, sqlalchemy.Integer)
+        assert isinstance(C.Attributes.sqla_table.c['d_id'].type,
+                                                             sqlalchemy.Integer)
 
     def test_append_field_array(self):
         class C(TableModel):
@@ -750,7 +1098,8 @@ class TestSqlAlchemySchema(unittest.TestCase):
         C.append_field('d', Array(D).store_as('table'))
         assert C.Attributes.sqla_mapper.get_property('d').argument is D
         print(repr(D.Attributes.sqla_table))
-        assert isinstance(D.Attributes.sqla_table.c['c_id'].type, sqlalchemy.Integer)
+        assert isinstance(D.Attributes.sqla_table.c['c_id'].type,
+                                                             sqlalchemy.Integer)
 
     def test_append_field_array_many(self):
         class C(TableModel):
@@ -910,7 +1259,7 @@ class TestSqlAlchemySchemaWithPostgresql(unittest.TestCase):
 
         t = self.metadata.tables[table_name]
         assert 'e' in t.c
-        assert t.c.e.type.enums == enums
+        assert tuple(t.c.e.type.enums) == enums
 
 
 if __name__ == '__main__':
